import bisect
import numbers
import operator
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools.abctools import AbjadObject


class BreakPointFunction(AbjadObject):
    '''A break-point function:

    ::

        >>> bpf = datastructuretools.BreakPointFunction({
        ...     0.:   0.,
        ...     0.75: (-1, 1.),
        ...     1.:   0.25,
        ... })

    Allows interpolated lookup, and supports discontiguities on the y-axis:

    ::

        >>> for x in (-0.5, 0., 0.25, 0.5, 0.7499, 0.75, 1., 1.5):
        ...     bpf[x]
        0.0
        0.0
        -0.3333333333333333
        -0.6666666666666666
        -0.9998666666666667
        1.0
        0.25
        0.25

    Return `BreakPointFunction` instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_bpf', '_dc_bias', '_x_values', '_y_range')

    ### INITIALIZER ###

    def __init__(self, bpf):
        if isinstance(bpf, type(self)):
            assert isinstance(bpf, type(self))
            new_bpf = bpf._bpf.copy()
        elif isinstance(bpf, dict):
            new_bpf = bpf.copy()
            for x, ys in new_bpf.iteritems():
                assert isinstance(x, numbers.Real)
                if isinstance(ys, (list, tuple)):
                    assert len(ys) in (1, 2)
                    assert all(isinstance(y, numbers.Real) for y in ys)
                    if len(ys) == 2 and ys[0] == ys[1]:
                        new_bpf[x] = (ys[0],)
                    else:
                        new_bpf[x] = tuple(ys)
                elif isinstance(ys, numbers.Real):
                    new_bpf[x] = (ys,)
        assert 1 < len(new_bpf)
        self._bpf = new_bpf
        self._update_caches()

    ### SPECIAL METHODS ###

    def __add__(self, other):
        '''Add `other` to all y-values in self:

        ::

            >>> bpf = datastructuretools.BreakPointFunction(
            ...     {0.: 0., 0.75: (-1., 1.), 1.: 0.25})
            >>> bpf + 0.3
            BreakPointFunction({
                0.0: (0.3,),
                0.75: (-0.7, 1.3),
                1.0: (0.55,)
            })

        `other` may also be a BreakPointFunction instance:

        ::

            >>> bpf2 = datastructuretools.BreakPointFunction({0.: 1., 1.: 0.})
            >>> bpf + bpf2
            BreakPointFunction({
                0.0: (1.0,),
                0.75: (-0.75, 1.25),
                1.0: (0.25,)
            })

        Emit new `BreakPointFunction`.
        '''
        return self._operate(other, operator.add)

    def __div__(self, other):
        '''Divide y-values in self by `other`:

        ::

            >>> bpf = datastructuretools.BreakPointFunction(
            ...     {0.: 0., 0.75: (-1., 1.), 1.: 0.25})
            >>> bpf / 2.
            BreakPointFunction({
                0.0: (0.0,),
                0.75: (-0.5, 0.5),
                1.0: (0.125,)
            })

        `other` may also be a `BreakPointFunction` instance.

        Emit new `BreakPointFunction`.
        '''
        return self._operate(other, operator.div)

    def __getitem__(self, item):
        '''Aliases BreakPointFunction.get_y_at_x().
        '''
        return self.get_y_at_x(item)

    def __mul__(self, other):
        '''Multiply y-values in self by `other`:

        ::

            >>> bpf = datastructuretools.BreakPointFunction(
            ...     {0.: 0., 0.75: (-1., 1.), 1.: 0.25})
            >>> bpf * 2.
            BreakPointFunction({
                0.0: (0.0,),
                0.75: (-2.0, 2.0),
                1.0: (0.5,)
            })

        `other` may also be a `BreakPointFunction` instance.

        Emit new `BreakPointFunction`.
        '''
        return self._operate(other, operator.mul)

    def __repr__(self):
        result = ['{}({{'.format(self._class_name)]
        if 1 < len(self.x_values):
            for x in self.x_values[:-1]:
                result.append('\t{!r}: {!r},'.format(x, self._bpf[x]))
        result.append('\t{!r}: {!r}'.format(self.x_values[-1], self._bpf[self.x_values[-1]]))
        result.append('})')
        return '\n'.join(result)

    def __sub__(self, other):
        '''Subtract `other` from all y-values in self:

        ::

            >>> bpf = datastructuretools.BreakPointFunction(
            ...     {0.: 0., 0.75: (-1., 1.), 1.: 0.25})
            >>> bpf - 0.3
            BreakPointFunction({
                0.0: (-0.3,),
                0.75: (-1.3, 0.7),
                1.0: (-0.04999999999999999,)
            })

        `other` may also be a BreakPointFunction instance:

        ::

            >>> bpf2 = datastructuretools.BreakPointFunction(
            ...     {0.: 1., 1.: 0.})
            >>> bpf - bpf2
            BreakPointFunction({
                0.0: (-1.0,),
                0.75: (-1.25, 0.75),
                1.0: (0.25,)
            })

        Emit new `BreakPointFunction`.
        '''
        return self._operate(other, operator.sub)

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def bpf(self):
        '''A copy of the BreakPointFunction's internal data-structure:

        ::

            >>> datastructuretools.BreakPointFunction(
            ...     {0.: 0.25, 0.5: 1.3, 1.: 0.9}).bpf
            {0.0: (0.25,), 0.5: (1.3,), 1.0: (0.9,)}

        Return dict.
        '''
        return self._bpf.copy()

    @property
    def dc_bias(self):
        '''The mean y-value of a BreakPointFunction:

        ::

            >>> datastructuretools.BreakPointFunction(
            ...     {0.: 0., 0.25: 0., 0.5: (0.75, 0.25), 1.: 1.}
            ...     ).dc_bias
            0.4

        Return number.
        '''
        return self._dc_bias

    @property
    def gnuplot_format(self):
        result = [
            "set border lw 1.5 lc rgb '#606060'",
            "set output {filename!r}",
            "set style line 1 lc rgb '#000000' lt 1 lw 2 pt 7 pi -1 ps 1.5",
            "set terminal {image_format} size {width},{height} enhanced",
            "set tics scale 0.75",
            "unset key",
            "plot '-' using 1:2 with linespoints ls 1",
        ]
        for x, ys in sorted(self._bpf.iteritems()):
            if len(ys) == 2:
                result.append('\t{} {}'.format(float(x), float(ys[0])))
                result.append('')
            result.append('\t{} {}'.format(float(x), float(ys[-1])))
        return '\n'.join(result)

    @property
    def x_center(self):
        '''The arithmetic mean of a BreakPointFunction's x-range:

        ::

            >>> datastructuretools.BreakPointFunction(
            ...     {0.: 0.25, 0.5: 1.3, 1.: 0.9}).x_center
            0.5

        Return number.
        '''
        return (self.x_range[1] + self.x_range[0]) / 2

    @property
    def x_range(self):
        '''The minimum and maximum x-values of a BreakPointFunction:

        ::

            >>> datastructuretools.BreakPointFunction(
            ...     {0.: 0.25, 0.5: 1.3, 1.: 0.9}).x_range
            (0.0, 1.0)

        Return pair.
        '''
        return self.x_values[0], self.x_values[-1]

    @property
    def x_values(self):
        '''The sorted x-values of a BreakPointFunction:

        ::

            >>> datastructuretools.BreakPointFunction(
            ...     {0.: 0.25, 0.5: 1.3, 1.: 0.9}).x_values
            (0.0, 0.5, 1.0)

        Return tuple.
        '''
        return tuple(sorted(self._bpf))

    @property
    def y_center(self):
        '''The arithmetic mean of a BreakPointFunction's y-range:

        ::

            >>> datastructuretools.BreakPointFunction(
            ...     {0.: 0.25, 0.5: 1.3, 1.: 0.9}).y_center
            0.775

        Return number.
        '''
        return (self.y_range[1] + self.y_range[0]) / 2

    @property
    def y_range(self):
        '''The minimum and maximum y-values of a BreakPointFunction:

        ::

            >>> datastructuretools.BreakPointFunction(
            ...     {0.: 0.25, 0.5: 1.3, 1.: 0.9}).y_range
            (0.25, 1.3)

        Return pair.
        '''
        return self._y_range

    ### PRIVATE METHODS ###

    def _operate(self, other, operator):
        assert isinstance(other, (type(self), numbers.Real))
        if isinstance(other, type(self)):
            x_values = sorted(self.x_values + other.x_values)
        else:
            x_values = self.x_values
        bpf = {}
        for x in x_values:
            if x in self._bpf:
                one_ys = self._bpf[x]
            else:
                one_ys = [self.get_y_at_x(x)]
            if isinstance(other, type(self)):
                if x in other._bpf:
                    two_ys = other._bpf[x]
                else:
                    two_ys = [other.get_y_at_x(x)]
            else:
                two_ys = [other]
            new_ys = []
            if len(one_ys) == len(two_ys):
                if len(one_ys) == 2:
                    new_ys.append(operator(one_ys[0], two_ys[0]))
                    new_ys.append(operator(one_ys[1], two_ys[1]))
                else:
                    new_ys.append(operator(one_ys[0], two_ys[0]))
            elif len(two_ys) < len(one_ys):
                new_ys.append(operator(one_ys[0], two_ys[0]))
                new_ys.append(operator(one_ys[1], two_ys[0]))
            else:
                new_ys.append(operator(two_ys[0], one_ys[0]))
                new_ys.append(operator(two_ys[1], one_ys[0]))
            bpf[x] = new_ys
        return type(self)(bpf)

    def _scale(self, value, old_min, old_max, new_min, new_max):
        old_range = old_max - old_min
        new_range = new_max - new_min
        return (((value - old_min) / old_range) * new_range) + new_min

    def _update_caches(self): 
        x_values = []
        y_values = []
        for x, ys in self._bpf.iteritems():
            x_values.append(x)
            y_values.extend(ys)
        self._dc_bias = sum(y_values) / len(y_values)
        self._x_values = tuple(sorted(x_values))
        self._y_range = (min(y_values), max(y_values))

    ### PUBLIC METHODS ###

    def clip_x_axis(self, minimum=0, maximum=1):
        '''Clip x-axis between `minimum` and `maximum`:

        ::

            >>> bpf = datastructuretools.BreakPointFunction(
            ...     {0.: 1., 1.: 0.})
            >>> bpf.clip_x_axis(minimum=0.25, maximum=0.75)
            BreakPointFunction({
                0.25: (0.75,),
                0.75: (0.25,)
            })

        Emit new `BreakPointFunction` instance.
        '''
        assert isinstance(minimum, numbers.Real)
        assert isinstance(maximum, numbers.Real)
        assert minimum < maximum
        bpf = {}
        x_values = [minimum]
        x_values.extend([x for x in self.x_values if minimum < x < maximum])
        x_values.append(maximum)
        for x_value in x_values:
            if x_value in self._bpf:
                bpf[x_value] = self._bpf[x_value]
            else:
                bpf[x_value] = (self[x_value],)
        return type(self)(bpf)

    def clip_y_axis(self, minimum=0, maximum=1):
        '''Clip y-axis between `minimum` and `maximum`:

        ::

            >>> bpf = datastructuretools.BreakPointFunction(
            ...     {0.: 1., 1.: 0.})
            >>> bpf.clip_y_axis(minimum=0.25, maximum=0.75)
            BreakPointFunction({
                0.0: (0.75,),
                1.0: (0.25,)
            })

        Emit new `BreakPointFunction` instance.
        '''
        assert isinstance(minimum, numbers.Real)
        assert isinstance(maximum, numbers.Real)
        assert minimum < maximum
        bpf = {}
        for x, ys in self._bpf.iteritems():
            new_ys = []
            for y in ys:
                if y < minimum:
                    new_ys.append(minimum)
                elif maximum < y:
                    new_ys.append(maximum)
                else:
                    new_ys.append(y)
            if len(new_ys) == 2 and new_ys[0] == new_ys[1]:
                new_ys.pop()
            bpf[x] = tuple(new_ys)
        return type(self)(bpf)

    def concatenate(self, other):
        '''Concatenate self with `other`:

        ::

            >>> one = datastructuretools.BreakPointFunction(
            ...     {0.0: 0.0, 1.0: 1.0})
            >>> two = datastructuretools.BreakPointFunction(
            ...     {0.5: 0.75, 1.5: 0.25})

        ::

            >>> one.concatenate(two)
            BreakPointFunction({
                0.0: (0.0,),
                1.0: (1.0, 0.75),
                2.0: (0.25,)
            })

        Emit new `BreakPointFunction` instance.
        '''
        assert isinstance(other, type(self))
        last_x_of_first_bpf = self.x_values[-1]
        first_x_of_second_bpf = other.x_values[0]
        x_shift = first_x_of_second_bpf - last_x_of_first_bpf
        stop_y = self._bpf[last_x_of_first_bpf][0]
        start_y = other._bpf[first_x_of_second_bpf][-1]
        hinge_ys = (stop_y, start_y) 
        new_bpf_dict = self.bpf
        new_bpf_dict[last_x_of_first_bpf] = hinge_ys
        for x in other.x_values[1:]:
            new_bpf_dict[x - x_shift] = other._bpf[x]
        return type(self)(new_bpf_dict) 

    def get_y_at_x(self, x):
        '''Get `y`-value at `x`:

        ::

            >>> bpf = datastructuretools.BreakPointFunction(
            ...     {0.: 0., 0.5: (-1., 1.), 1.: 0.5})

        ::

            >>> bpf.get_y_at_x(-1000)
            0.0

        ::

            >>> bpf.get_y_at_x(0.)
            0.0

        ::

            >>> bpf.get_y_at_x(0.25)
            -0.5

        ::

            >>> bpf.get_y_at_x(0.4999)
            -0.9998

        ::

            >>> bpf.get_y_at_x(0.5)
            1.0

        ::

            >>> bpf.get_y_at_x(0.75)
            0.75

        ::

            >>> bpf.get_y_at_x(1.)
            0.5

        ::

            >>> bpf.get_y_at_x(1000)
            0.5

        Return Number.
        '''
        if x <= self.x_values[0]:
            return self._bpf[self.x_values[0]][0]
        elif self.x_values[-1] <= x:
            return self._bpf[self.x_values[-1]][-1]
        elif x in self.x_values:
            return self._bpf[x][-1]
        idx = bisect.bisect(self.x_values, x)
        x0 = self.x_values[idx - 1]
        x1 = self.x_values[idx]
        y0 = self._bpf[x0][-1]
        y1 = self._bpf[x1][0]
        dx = x1 - x0
        dy = y1 - y0
        m = float(dy) / float(dx)
        b = y0 - (m * x0) 
        return (x * m) + b

    def invert(self, y_center=None):
        '''Invert self:

        ::

            >>> datastructuretools.BreakPointFunction(
            ...     {0.: 0., 1.: 1.}).invert()
            BreakPointFunction({
                0.0: (1.0,),
                1.0: (0.0,)
            })

        If `y_center` is not None, use `y_center` as the axis of inversion:

        ::

            >>> datastructuretools.BreakPointFunction(
            ...     {0.: 0., 1.: 1.}).invert(0)
            BreakPointFunction({
                0.0: (0.0,),
                1.0: (-1.0,)
            })

        ::

            >>> datastructuretools.BreakPointFunction(
            ...     {0.: 0., 1.: 1.}).invert(0.25)
            BreakPointFunction({
                0.0: (0.5,),
                1.0: (-0.5,)
            })

        Emit new `BreakPointFunction` instance.
        '''
        if y_center is None:
            y_center = self.y_center
        else:
            assert isinstance(y_center, numbers.Real)
        bpf = {}
        for x, ys in self._bpf.iteritems():
            new_ys = [((y_center - y) + y_center) for y in ys]
            bpf[x] = tuple(new_ys)
        return type(self)(bpf)

    def normalize_axes(self):
        '''Scale both x and y axes between 0 and 1:

        ::

            >>> datastructuretools.BreakPointFunction(
            ...     {0.25: 0.25, 0.75: 0.75}).normalize_axes()
            BreakPointFunction({
                0.0: (0.0,),
                1.0: (1.0,)
            })

        Emit new `BreakPointFunction` instance.
        '''
        return self.scale_x_axis().scale_y_axis()

    def reflect(self, x_center=None):
        '''Reflect x values of a `BreakPointFunction`:

        ::

            >>> datastructuretools.BreakPointFunction(
            ...     {0.25: 0., 0.5: (-1., 2.), 1: 1.}).reflect()
            BreakPointFunction({
                0.25: (1.0,),
                0.75: (2.0, -1.0),
                1.0: (0.0,)
            })

        If `x_center` is not None, reflection will take `x_center` as the axis of reflection:

        ::

            >>> datastructuretools.BreakPointFunction(
            ...     {0.25: 0., 0.5: (-1., 2.), 1: 1.}).reflect(x_center=0.25)
            BreakPointFunction({
                -0.5: (1.0,),
                0.0: (2.0, -1.0),
                0.25: (0.0,)
            })

        Emit new `BreakPointFunction` instance.
        '''
        bpf = {}
        if x_center is None:
            x_center = self.x_center
        else:
            assert isinstance(x_center, numbers.Real)
        for x, ys in self._bpf.iteritems():
            new_x = (x_center - x) + x_center
            bpf[new_x] = tuple(reversed(ys))
        return type(self)(bpf)

    def remove_dc_bias(self):
        '''Remove dc-bias from a `BreakPointFunction`:

        ::

            >>> datastructuretools.BreakPointFunction(
            ...     {0.: 0., 1.: 1.}).remove_dc_bias()
            BreakPointFunction({
                0.0: (-0.5,),
                1.0: (0.5,)
            })

        Emit new `BreakPointFunction` instance.
        '''
        return self - self.dc_bias

    def scale_x_axis(self, minimum=0, maximum=1):
        '''Scale x-axis between `minimum` and `maximum`:

        ::

            >>> datastructuretools.BreakPointFunction(
            ...     {0.: 0., 0.5: (-1., 2.), 1.: 1.}
            ...     ).scale_x_axis(-2, 2)
            BreakPointFunction({
                -2.0: (0.0,),
                0.0: (-1.0, 2.0),
                2.0: (1.0,)
            })

        Emit new `BreakPointFunction` instance.
        '''
        assert isinstance(minimum, numbers.Real)
        assert isinstance(maximum, numbers.Real)
        assert minimum < maximum
        bpf = {}
        x_min, x_max = self.x_range 
        for x, ys in self._bpf.iteritems():
            bpf[self._scale(x, x_min, x_max, minimum, maximum)] = ys
        return type(self)(bpf)

    def scale_y_axis(self, minimum=0, maximum=1):
        '''Scale y-axis between `minimum` and `maximum`:

        ::

            >>> datastructuretools.BreakPointFunction(
            ...     {0.: 0., 0.5: (-1., 2.), 1.: 1.}
            ...     ).scale_y_axis(-2, 4)
            BreakPointFunction({
                0.0: (0.0,),
                0.5: (-2.0, 4.0),
                1.0: (2.0,)
            })

        Emit new `BreakPointFunction` instance.
        '''
        assert isinstance(minimum, numbers.Real)
        assert isinstance(maximum, numbers.Real)
        assert minimum < maximum
        bpf = {}
        y_min, y_max = self.y_range
        for x, ys in self._bpf.iteritems():
            bpf[x] = tuple([self._scale(y, y_min, y_max, minimum, maximum) for y in ys])
        return type(self)(bpf)

    def set_y_at_x(self, x, y):
        '''Set `y`-value at `x`:

        ::

            >>> bpf = datastructuretools.BreakPointFunction(
            ...     {0.0: 0.0, 1.0: 1.0})

        With a number:

        ::

            >>> bpf.set_y_at_x(0.25, 0.75)
            >>> bpf
            BreakPointFunction({
                0.0: (0.0,),
                0.25: (0.75,),
                1.0: (1.0,)
            })

        With a pair:

        ::

            >>> bpf.set_y_at_x(0.6, (-2., 2.))
            >>> bpf
            BreakPointFunction({
                0.0: (0.0,),
                0.25: (0.75,),
                0.6: (-2.0, 2.0),
                1.0: (1.0,)
            })

        Delete all values at `x` when `y` is None:

        ::

            >>> bpf.set_y_at_x(0., None)
            >>> bpf
            BreakPointFunction({
                0.25: (0.75,),
                0.6: (-2.0, 2.0),
                1.0: (1.0,)
            })

        Operate in place and return None.
        '''
        assert isinstance(x, numbers.Real)
        if isinstance(y, numbers.Real):
            self._bpf[x] = (y,)
        elif isinstance(y, (list, tuple)):
            assert len(y) in (1, 2) and all(isinstance(j, numbers.Real) for j in y)
            if len(y) == 2 and y[0] == y[1]:
                self._bpf[x] = (y[0],)
            else:
                self._bpf[x] = tuple(y)
        elif isinstance(y, type(None)):
            if x in self._bpf:
                del(self._bpf[x])
        else:
            raise ValueError
        self._update_caches()
 
    def tessalate_by_ratio(self, ratio, invert_on_negative=False, reflect_on_negative=False,
        y_center=None):
        '''Concatenate copies of a BreakPointFunction, stretched by the weights in `ratio`:

        ::

            >>> bpf = datastructuretools.BreakPointFunction(
            ...     {0.: 0., 0.25: 0.9, 1.: 1.})

        ::

            >>> bpf.tessalate_by_ratio((1, 2, 3))
            BreakPointFunction({
                0.0: (0.0,),
                0.25: (0.9,),
                1.0: (1.0, 0.0),
                1.5: (0.9,),
                3.0: (1.0, 0.0),
                3.75: (0.9,),
                6.0: (1.0,)
            })

        Negative ratio values are still treated as weights.

        If `invert_on_negative` is True, copies corresponding to negative ratio values
        will be inverted:

        ::

            >>> bpf.tessalate_by_ratio((1, -2, 3), invert_on_negative=True)
            BreakPointFunction({
                0.0: (0.0,),
                0.25: (0.9,),
                1.0: (1.0,),
                1.5: (0.09999999999999998,),
                3.0: (0.0,),
                3.75: (0.9,),
                6.0: (1.0,)
            })

        If `y_center` is not None, inversion will take `y_center` as the axis of inversion:

        ::

            >>> bpf.tessalate_by_ratio((1, -2, 3), invert_on_negative=True, y_center=0)
            BreakPointFunction({
                0.0: (0.0,),
                0.25: (0.9,),
                1.0: (1.0, 0.0),
                1.5: (-0.9,),
                3.0: (-1.0, 0.0),
                3.75: (0.9,),
                6.0: (1.0,)
            })

        If `reflect_on_negative` is True, copies corresponding to negative ratio values
        will be reflectd:

        ::

            >>> bpf.tessalate_by_ratio((1, -2, 3), reflect_on_negative=True)
            BreakPointFunction({
                0.0: (0.0,),
                0.25: (0.9,),
                1.0: (1.0,),
                2.5: (0.9,),
                3.0: (0.0,),
                3.75: (0.9,),
                6.0: (1.0,)
            })

        Inversion may be combined reflecting.

        Emit new `BreakPointFunction` instance.
        '''
        ratio = mathtools.Ratio(ratio)
        tessalated_bpf = None
        for i, pair in enumerate(mathtools.cumulative_sums_zero_pairwise(
            [abs(x) for x in ratio])):
            sign = mathtools.sign(ratio[i])
            start, stop = pair
            bpf = self.scale_x_axis(start, stop)
            if sign < 0:
                if invert_on_negative:
                    bpf = bpf.invert(y_center)
                if reflect_on_negative:
                    bpf = bpf.reflect()
            if i == 0:
                tessalated_bpf = bpf
            else:
                tessalated_bpf = tessalated_bpf.concatenate(bpf)
        return tessalated_bpf  
            



def transpose_chromatic_pitch_by_melodic_chromatic_interval_segment(pitch, segment):
    '''.. versionadded:: 2.0

    Transpose chromatic `pitch` by melodic chromatic interval `segment`::

        >>> ncp = pitchtools.NumberedChromaticPitch(0)
        >>> mcis = pitchtools.MelodicChromaticIntervalSegment([0, -1, 2])
        >>> pitchtools.transpose_chromatic_pitch_by_melodic_chromatic_interval_segment(ncp, mcis)
        [NumberedChromaticPitch(0), NumberedChromaticPitch(-1), NumberedChromaticPitch(1)]

    Transpose by each interval in `segment` such that each tranposition
    transposes the resulting pitch of the previous transposition.

    Return list of numbered chromatic pitches.
    '''
    from abjad.tools import pitchtools

    # check input
    if not isinstance(pitch, pitchtools.ChromaticPitchObject):
        raise TypeError

    # check input
    if not isinstance(segment, pitchtools.MelodicChromaticIntervalSegment):
        raise TypeError

    if not hasattr(pitch, 'transpose'):
        pitch = pitch.numbered_chromatic_pitch

    pitches = [pitch.transpose(segment[0].number)]

    for interval in segment[1:]:
        pitches.append(pitches[-1].transpose(interval.number))

    return pitches



from abjad import *
from abjad.tools import iotools


def test_stringtools_underscore_delimited_lowercase_to_lowercamelcase_01():

    assert stringtools.underscore_delimited_lowercase_to_lowercamelcase('foo_bar_blah') == 'fooBarBlah'



from abjad.tools import iotools
from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript
import argparse
import os
import subprocess


class SvnAddAllScript(DirectoryScript):
    '''Run `svn add` on all unversioned files in path:

    ::

        bash$ ajv svn add -h
        usage: svn-add-all [-h] [--version] path

        "svn add" all unversioned files in PATH.

        positional arguments:
          path        directory tree to be recursed over

        optional arguments:
          -h, --help  show this help message and exit
          --version   show program's version number and exit

    Return `SvnAddAllScript` instance.
    '''

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def alias(self):
        return 'add'

    @property
    def long_description(self):
        return None

    @property
    def scripting_group(self):
        return 'svn'

    @property
    def short_description(self):
        return '"svn add" all unversioned files in PATH.'

    @property
    def version(self):
        return 1.0

    ### PUBLIC METHODS ###

    def process_args(self, args):

        command = 'svn st {}'.format(args.path)

        process = subprocess.Popen(command,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
            )

        lines = process.stdout.readlines()

        for line in lines:
            if line.startswith('?'):
                command = 'svn add {}'.format(line.split()[-1])
                iotools.spawn_subprocess(command)

    def setup_argument_parser(self, parser):
        parser.add_argument('path',
            type=self._validate_path,
            help='directory tree to be recursed over'
            )



from abjad.tools import durationtools


def calculate_min_mean_and_max_durations_of_intervals(intervals):
    '''Return a 3-tuple of the minimum, mean and maximum duration of all intervals in `intervals`.
    If `intervals` is empty, return None.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    if not tree:
        return None

    durations = [x.duration for x in tree]
    return (min(durations), durationtools.Duration(sum(durations), len(durations)), max(durations))



from abjad.tools import durationtools


def calculate_depth_centroid_of_intervals(intervals):
    '''Calculate the weighted mean offset of `intervals`, such that the
    centroids of each interval in the depth tree of `intervals` make up
    the values of the mean, and the depth of each interval in the depth
    tree of `intervals` make up the weights.

    Return Offset.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    if not tree:
        return None

    depth = timeintervaltools.compute_depth_of_intervals(tree)
    weighted_centroids = sum([x.center * x['depth'] for x in depth])
    sum_of_weights = sum([x['depth'] for x in depth])
    if not sum_of_weights:
        return None
    return durationtools.Offset(weighted_centroids) / sum_of_weights



from abjad import *
from abjad.tools.mathtools import NonreducedFraction


def test_durationtools_Duration___mul___01():
    '''Multiplying duration by nonreduced fraction gives nonreduced fraction.
    '''

    result = Duration(1) * NonreducedFraction(3, 3)
    assert isinstance(result, NonreducedFraction)
    assert result.pair == (3, 3)

    result = NonreducedFraction(3, 3) * Duration(1)
    assert isinstance(result, NonreducedFraction)
    assert result.pair == (3, 3)



def get_components_in_expr_with_name(expr, name):
    r'''.. versionadded:: 2.9

    Get components in `expr` with `name`::

        >>> staff = Staff(r"\new Voice { c'8 d'8 } \new Voice { e'8 f'8 } \new Voice { g'4 }")
        >>> staff[0].name = 'outer voice'
        >>> staff[1].name = 'middle voice'
        >>> staff[2].name = 'outer voice'

        >>> f(staff)
        \new Staff {
            \context Voice = "outer voice" {
                c'8
                d'8
            }
            \context Voice = "middle voice" {
                e'8
                f'8
            }
            \context Voice = "outer voice" {
                g'4
            }
        }

        >>> componenttools.get_components_in_expr_with_name(staff, 'outer voice')
        [Voice-"outer voice"{2}, Voice-"outer voice"{1}]

    ::

        >>> componenttools.get_components_in_expr_with_name(staff, 'middle voice')
        [Voice-"middle voice"{2}]

    Return list of zero or more components found.
    '''
    from abjad.tools import iterationtools

    result = []
    
    for component in iterationtools.iterate_components_in_expr(expr):
        if name is None or getattr(component, 'name', None) == name:
            result.append(component)

    return result



from abjad.tools import durationtools
from abjad.tools import mathtools


def make_tied_leaf(kind, duration, decrease_durations_monotonically=True, pitches=None, tie_parts=True):
    '''.. versionadded:: 1.0

    Make tied leaf of `kind` with assignable `duration`.

    `decrease_durations_monotonically` must be boolean.
        True returns a list of notes of decreasing duration.
        False returns a list of notes of increasing duration.

    `pitches`
        a pitch or list of pitches.

    `tie_parts`
        True to return tied leaves. False otherwise.
    '''
    from abjad.tools import tietools

    # check input
    duration = durationtools.Duration(duration)

    # make leaves
    result = []
    for numerator in mathtools.partition_integer_into_canonic_parts(duration.numerator):
        written_duration = durationtools.Duration(numerator, duration.denominator)
        if not pitches is None:
            args = (pitches, written_duration)
        else:
            args = (written_duration, )
        result.append(kind(*args))
    
    # apply direction and tie spanner if required
    if 1 < len(result):
        if not decrease_durations_monotonically:
            result.reverse()
        if tie_parts:
            tietools.TieSpanner(result)

    # return result
    return result



def partition_components_by_durations_not_greater_than(components, durations,     
    cyclic=False, in_seconds=False, overhang=False):
    r'''.. versionadded:: 1.1
    '''
    from abjad.tools.componenttools._partition_components_by_durations import _partition_components_by_durations

    if in_seconds:
        duration_type = 'seconds'
    else:
        duration_type = 'prolated'

    return _partition_components_by_durations(duration_type, components, durations,
        fill='less', cyclic=cyclic, overhang=overhang)



from abjad.tools import componenttools
from abjad.tools import durationtools
from abjad.tools import leaftools
from abjad.tools import mathtools


def leaf_to_tuplet_with_ratio(leaf, proportions, is_diminution=True):
    '''Change `leaf` to tuplet with `proportions`.

    Example 1. Change `leaf` to augmented tuplet with `proportions`::

        >>> note = Note(0, (3, 16))

    ::

        >>> print tuplettools.leaf_to_tuplet_with_ratio(
        ...     note, [1], is_diminution=False)
        {@ 1:1 c'8. @}

    ::

        >>> print tuplettools.leaf_to_tuplet_with_ratio(
        ...     note, [1, 2], is_diminution=False)
        {@ 1:1 c'16, c'8 @}

    ::

        >>> print tuplettools.leaf_to_tuplet_with_ratio(
        ...     note, [1, 2, 2], is_diminution=False)
        {@ 5:8 c'64., c'32., c'32. @}

    ::

        >>> print tuplettools.leaf_to_tuplet_with_ratio(
        ...     note, [1, 2, 2, 3], is_diminution=False)
        {@ 2:3 c'64, c'32, c'32, c'32. @}

    ::

        >>> print tuplettools.leaf_to_tuplet_with_ratio(
        ...     note, [1, 2, 2, 3, 3], is_diminution=False)
        {@ 11:12 c'64, c'32, c'32, c'32., c'32. @}

    ::

        >>> print tuplettools.leaf_to_tuplet_with_ratio(
        ...     note, [1, 2, 2, 3, 3, 4], is_diminution=False)
        {@ 5:8 c'128, c'64, c'64, c'64., c'64., c'32 @}

    Example 2. Change `leaf` to diminished tuplet::

        >>> note = Note(0, (3, 16))

    ::

        >>> print tuplettools.leaf_to_tuplet_with_ratio(
        ...     note, [1], is_diminution=True)
        {@ 1:1 c'8. @}

    ::

        >>> print tuplettools.leaf_to_tuplet_with_ratio(
        ...     note, [1, 2], is_diminution=True)
        {@ 1:1 c'16, c'8 @}

    ::

        >>> print tuplettools.leaf_to_tuplet_with_ratio(
        ...     note, [1, 2, 2], is_diminution=True)
        {@ 5:4 c'32., c'16., c'16. @}

    ::

        >>> print tuplettools.leaf_to_tuplet_with_ratio(
        ...     note, [1, 2, 2, 3], is_diminution=True)
        {@ 4:3 c'32, c'16, c'16, c'16. @}

    ::

        >>> print tuplettools.leaf_to_tuplet_with_ratio(
        ...     note, [1, 2, 2, 3, 3], is_diminution=True)
        {@ 11:6 c'32, c'16, c'16, c'16., c'16. @}

    ::

        >>> print tuplettools.leaf_to_tuplet_with_ratio(
        ...     note, [1, 2, 2, 3, 3, 4], is_diminution=True)
        {@ 5:4 c'64, c'32, c'32, c'32., c'32., c'16 @}

    Return fixed-duration tuplet.
    '''
    from abjad.tools import notetools
    from abjad.tools import tuplettools

    # check input
    assert isinstance(leaf, leaftools.Leaf)
    proportions = mathtools.Ratio(proportions)

    # find target duration of fixed-duration tuplet
    target_duration = leaf.written_duration

    # find basic prolated duration of note in tuplet
    basic_prolated_duration = target_duration / sum(proportions)

    # find basic written duration of note in tuplet
    basic_written_duration = basic_prolated_duration.equal_or_greater_assignable

    # find written duration of each note in tuplet
    written_durations = [x * basic_written_duration for x in proportions]

    # make tuplet notes
    try:
        notes = [notetools.Note(0, x) for x in written_durations]
    except AssignabilityError:
        denominator = target_duration._denominator
        note_durations = [durationtools.Duration(x, denominator) for x in proportions]
        notes = notetools.make_notes(0, note_durations)

    # make tuplet
    tuplet = tuplettools.FixedDurationTuplet(target_duration, notes)

    # fix tuplet contents if necessary
    tuplettools.fix_contents_of_tuplets_in_expr(tuplet)

    # switch prolation if necessary
    if not tuplet.multiplier == 1:
        if is_diminution:
            if not tuplet.is_diminution:
                tuplettools.change_augmented_tuplets_in_expr_to_diminished(tuplet)
        else:
            if tuplet.is_diminution:
                tuplettools.change_diminished_tuplets_in_expr_to_augmented(tuplet)

    # give leaf position in score structure to tuplet
    componenttools.move_parentage_and_spanners_from_components_to_components([leaf], [tuplet])

    # return tuplet
    return tuplet



from abjad import *
from abjad.tools.mathtools import NonreducedFraction


def test_durationtools_Duration___div___01():
    '''Dividing duration by nonreduced fraction gives nonreduced fraction.
    '''

    result = Duration(1) / NonreducedFraction(3, 3)
    assert isinstance(result, NonreducedFraction)
    assert result.pair == (3, 3)

    result = NonreducedFraction(3, 3) / Duration(1)
    assert isinstance(result, NonreducedFraction)
    assert result.pair == (3, 3)



from abjad.tools import componenttools


def iterate_components_in_spanner(spanner, klass=None, reverse=False):
    '''.. versionadded:: 2.10

    Yield components in `spanner` one at a time from left to right::

        >>> t = Staff("c'8 d'8 e'8 f'8")
        >>> p = beamtools.BeamSpanner(t[2:])

    ::

        >>> notes = spannertools.iterate_components_in_spanner(p, klass=Note)

    ::

        >>> for note in notes:
        ...   note
        Note("e'8")
        Note("f'8")

    Yield components in `spanner` one at a time from right to left::

    ::

        >>> notes = spannertools.iterate_components_in_spanner(p, klass=Note, reverse=True)

    ::

        >>> for note in notes:
        ...   note
        Note("f'8")
        Note("e'8")

    Return generator.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import spannertools

    if not isinstance(spanner, spannertools.Spanner):
        raise TypeError

    klass = klass or componenttools.Component

    if not reverse:
        for component in spanner._components:
            dfs = iterationtools.iterate_components_depth_first(component)
            for node in dfs:
                if isinstance(node, klass):
                    yield node
    else:
        for component in reversed(spanner._components):
            dfs = iterationtools.iterate_components_depth_first(component, direction=Right)
            for node in dfs:
                if isinstance(node, klass):
                    yield node



from abjad.tools import componenttools


def move_parentage_children_and_spanners_from_components_to_empty_container(components, container):
    r'''Move parentage, children and spanners from donor `components` 
    to recipient empty `container`::

        >>> voice = Voice("{ c'8 [ d'8 } { e'8 f'8 } { g'8 a'8 ] }")

    ::

        >>> f(voice)
        \new Voice {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8 ]
            }
        }

    ::

        >>> tuplet = Tuplet(Fraction(3, 4), [])
        >>> containertools.move_parentage_children_and_spanners_from_components_to_empty_container(
        ... voice[:2], tuplet)

    ::

        >>> f(voice)
        \new Voice {
            \fraction \times 3/4 {
                c'8 [
                d'8
                e'8
                f'8
            }
            {
                g'8
                a'8 ]
            }
        }


    Return none.
    '''
    from abjad.tools import containertools
    from abjad.tools.componenttools._give_donor_components_position_in_parent_to_recipient_components import \
        _give_donor_components_position_in_parent_to_recipient_components
    from abjad.tools.componenttools._give_music_from_donor_components_to_recipient_components import \
        _give_music_from_donor_components_to_recipient_components
    from abjad.tools.spannertools._give_spanners_that_dominate_donor_components_to_recipient_components import \
        _give_spanners_that_dominate_donor_components_to_recipient_components

    assert componenttools.all_are_contiguous_components_in_same_parent(components)

    if not isinstance(container, containertools.Container):
        raise TypeError

    if not len(container) == 0:
        raise MusicContentsError

    _give_music_from_donor_components_to_recipient_components(components, container)
    _give_spanners_that_dominate_donor_components_to_recipient_components(components, [container])
    _give_donor_components_position_in_parent_to_recipient_components(components, [container])



from abjad.tools import documentationtools
from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript
import argparse
import importlib
import os


class CountToolsScript(DirectoryScript):
    '''Count public and private functions and classes in a path:

    ::

        bash$ ajv count tools -h
        usage: count-tools [-h] [--version] path

        Count tools in PATH.

        positional arguments:
          path        directory tree to be recursed over

        optional arguments:
          -h, --help  show this help message and exit
          --version   show program's version number and exit

    Return `CountToolsScript` instance.
    '''

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def alias(self):
        return 'tools'

    @property
    def long_description(self):
        return None

    @property
    def scripting_group(self):
        return 'count'

    @property
    def short_description(self):
        return 'Count tools in PATH.'

    @property
    def version(self):
        return 1.0

    ### PUBLIC METHODS ###

    def process_args(self, args):

        private_classes = []
        private_functions = []
        public_classes = []
        public_functions = []

        for module in documentationtools.ModuleCrawler(args.path):
            module_filename = module.__file__
            if module_filename.endswith('.pyc'):
                module_filename = module_filename[:-1]
            with open(module_filename, 'r') as f:
                lines = f.readlines()
                for line in lines:
                    if line.startswith('def '):
                        parts = line.split()
                        if parts[1].startswith('_'):
                            private_functions.append(line)
                        else:
                            public_functions.append(line)
                    elif line.startswith('class '):
                        parts = line.split()
                        if parts[1].startswith('_'):
                            private_classes.append(line)
                        else:
                            public_classes.append(line)

        print 'PUBLIC FUNCTIONS:  {}'.format(len(public_functions))
        print 'PUBLIC CLASSES:    {}'.format(len(public_classes))
        print 'PRIVATE FUNCTIONS: {}'.format(len(private_functions))
        print 'PRIVATE CLASSES:   {}'.format(len(private_classes))

    def setup_argument_parser(self, parser):
        parser.add_argument('path',
            type=self._validate_path,
            help='directory tree to be recursed over'
            )




import abc
from abjad.tools.pitchtools.ObjectSet import ObjectSet


class IntervalObjectSet(ObjectSet):
    '''.. versionadded:: 2.0

    Abstract interval set.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta

    ### INITIALIZER ###

    @abc.abstractmethod
    def __new__(self):
        pass



def list_known_contexts():
    '''List all LilyPond contexts recognized by ``LilyPondParser``:

    ::

        >>> for x in lilypondparsertools.list_known_contexts():
        ...     print x
        ...
        ChoirStaff
        ChordNames
        CueVoice
        Devnull
        DrumStaff
        DrumVoice
        Dynamics
        FiguredBass
        FretBoards
        Global
        GrandStaff
        GregorianTranscriptionStaff
        GregorianTranscriptionVoice
        KievanStaff
        KievanVoice
        Lyrics
        MensuralStaff
        MensuralVoice
        NoteNames
        PetrucciStaff
        PetrucciVoice
        PianoStaff
        RhythmicStaff
        Score
        Staff
        StaffGroup
        TabStaff
        TabVoice
        VaticanaStaff
        VaticanaVoice
        Voice

    Return list.
    '''
    from abjad.ly import contexts

    return sorted(contexts.keys())



from abjad.tools import beamtools
from abjad.tools import durationtools
from abjad.tools import iterationtools
from abjad.tools import spannertools
from abjad.tools.wellformednesstools.Check import Check


class BeamedQuarterNoteCheck(Check):

    def _run(self, expr):
        violators = []
        total = 0
        for leaf in iterationtools.iterate_leaves_in_expr(expr):
            total += 1
            if beamtools.is_component_with_beam_spanner_attached(leaf):
                beam = beamtools.get_beam_spanner_attached_to_component(leaf)
                if not isinstance(beam, beamtools.DuratedComplexBeamSpanner):
                    flag_count = leaf.written_duration.flag_count
                    if flag_count < 1:
                        violators.append(leaf)
        return violators, total



from abjad import *


def test_Spanner_format_01():
    '''Base Spanner class makes no format-time contributions.
    However, base spanner causes no explosions at format-time, either.
    '''

    class MockSpanner(spannertools.Spanner):
        def __init__(self, components=None):
            spannertools.Spanner.__init__(self, components)
        def _copy_keyword_args(self, new):
            pass

    t = Staff("c'8 d'8 e'8 f'8")
    p = MockSpanner(t[:])

    r'''
    \new Staff {
        c'8
        d'8
        e'8
        f'8
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Staff {\n\tc'8\n\td'8\n\te'8\n\tf'8\n}"



import os
import sys


# TODO: make public and possibly improve function name
def _run_abjad():
    from abjad.tools import iotools

    try:
        file = sys.argv[1]
    except IndexError:
        file = ''

    commands = [
        "from abjad import *;",
        "from abjad.tools import configurationtools;",
        "print configurationtools.get_abjad_startup_string();",
        "del configurationtools",
    ]

    command = '''python -i {} -c "{}"'''.format(file, ' '.join(commands))

    iotools.spawn_subprocess(command)



def all_are_grace_containers(expr, kind=None):
    r'''.. versionadded:: 2.6

    True when `expr` is a sequence of Abjad grace containers::

        >>> graces = [
        ...     gracetools.GraceContainer("<c' e' g'>4"), 
        ...     gracetools.GraceContainer("<c' f' a'>4")]
        >>> voice = Voice("c'8 d'8 e'8 f'8")
        >>> grace_notes = [Note("c'16"), Note("d'16")]
        >>> grace_container = gracetools.GraceContainer(grace_notes, kind='grace')
        >>> grace_container(voice[1])
        Note("d'8")

    ::

        >>> f(voice)
        \new Voice {
            c'8
            \grace {
                c'16
                d'16
            }
            d'8
            e'8
            f'8
        }

    ::

        >>> gracetools.all_are_grace_containers([grace_container])
        True

    True when `expr` is an empty sequence::

        >>> gracetools.all_are_grace_containers([])
        True

    Otherwise false::

        >>> gracetools.all_are_grace_containers('foo')
        False

    Return boolean.
    '''
    from abjad.tools import gracetools

    for element in expr:
        if not isinstance(element, gracetools.GraceContainer):
            return False
        if kind is not None and not element.kind == kind:
            return False
    return True



from abjad import *


def test_ComplexGlissandoSpanner___init___01():
    '''Init empty glissando spanner.
    '''

    glissando = spannertools.ComplexGlissandoSpanner()
    assert isinstance(glissando, spannertools.ComplexGlissandoSpanner)



from abjad import *


def test_tietools_remove_nonfirst_leaves_in_tie_chain_01():
    '''Keep and unspan first note in tie chain only.
    '''

    t = Staff(notetools.make_notes(0, [(5, 16)]))

    r'''
    \new Staff {
        c'4 ~
        c'16
    }
    '''

    tietools.remove_nonfirst_leaves_in_tie_chain(tietools.get_tie_chain(t[0]))

    r'''
    \new Staff {
        c'4
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Staff {\n\tc'4\n}"


def test_tietools_remove_nonfirst_leaves_in_tie_chain_02():
    '''Do not change trivial tie chains.
    '''

    t = Staff(notetools.make_repeated_notes(1))

    tietools.remove_nonfirst_leaves_in_tie_chain(tietools.get_tie_chain(t[0]))

    r'''
    \new Staff {
        c'8
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Staff {\n\tc'8\n}"



from abjad.tools import iotools
from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript
import doctest
import importlib
import os


class RunDoctestsScript(DirectoryScript):
    '''Run doctests on all Python files in current directory recursively:

    ::

        bash$ ajv doctest -h
        usage: run-doctests [-h] [--version]

        Run doctests on all modules in current path.

        optional arguments:
          -h, --help  show this help message and exit
          --version   show program's version number and exit

    Return `RunDoctestsScript` instance.
    '''

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def alias(self):
        return 'doctest'

    @property
    def long_description(self):
        return None

    @property
    def scripting_group(self):
        return None

    @property
    def short_description(self):
        return 'Run doctests on all modules in current path.'

    @property
    def version(self):
        return 1.0

    ### PUBLIC PROPERTIES ###

    def process_args(self, args):
        globs = importlib.import_module('abjad').__dict__.copy()
        try:
            globs.update(importlib.import_module('experimental.tools').__dict__)
            globs.update(importlib.import_module('experimental.demos').__dict__)
        except ImportError:
            pass
        if args.diff:
            optionflags = doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS|doctest.REPORT_NDIFF
        else:
            optionflags = doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS
        iotools.clear_terminal()
        total_modules = 0
        for dir_path, dir_names, file_names in os.walk('.'):
            for file_name in file_names:
                if file_name.endswith('.py') and \
                    not file_name.startswith('test_') and \
                    not file_name == '__init__.py':
                    total_modules += 1
                    full_file_name = os.path.abspath(os.path.join(dir_path, file_name))
                    doctest.testfile(full_file_name, module_relative=False, globs=globs,
                       optionflags=optionflags)
        print 'Total modules: %s' % total_modules

    def setup_argument_parser(self, parser):

        parser.add_argument('--diff',
            action='store_true',
            help='print diff-like output on failed tests.',
            )



from abjad.tools import componenttools
from abjad.tools import containertools


def move_prolation_of_tuplet_to_contents_of_tuplet_and_remove_tuplet(tuplet):
    r'''Move prolation of `tuplet` to contents of `tuplet` and remove `tuplet`::

        >>> t = Staff(r"\times 3/2 { c'8 [ d'8 } \times 3/2 { c'8 d'8 ] }")

    ::

        >>> f(t)
        \new Staff {
            \fraction \times 3/2 {
                c'8 [
                d'8
            }
            \fraction \times 3/2 {
                c'8
                d'8 ]
            }
        }

    ::

        >>> tuplettools.move_prolation_of_tuplet_to_contents_of_tuplet_and_remove_tuplet(t[0])
        Tuplet(3/2, [])

    ::

        >>> f(t)
        \new Staff {
            c'8. [
            d'8.
            \fraction \times 3/2 {
                c'8
                d'8 ]
            }
        }

    Return `tuplet`.
    '''
    from abjad.tools import tuplettools

    assert isinstance(tuplet, tuplettools.Tuplet)

    containertools.scale_contents_of_container(tuplet, tuplet.multiplier)
    componenttools.move_parentage_and_spanners_from_components_to_components([tuplet], tuplet[:])

    return tuplet



from abjad.tools.rhythmmakertools.OutputIncisedRhythmMaker import OutputIncisedRhythmMaker


class OutputIncisedNoteRhythmMaker(OutputIncisedRhythmMaker):
    r'''.. versionadded:: 2.8
    
    Output-incised note rhythm-maker.

    Configure the rhythm-maker on initialization::

        >>> prefix_talea, prefix_lengths = [-8], [2]
        >>> suffix_talea, suffix_lengths = [-3], [4]
        >>> talea_denominator = 32
        >>> maker = rhythmmakertools.OutputIncisedNoteRhythmMaker(
        ... prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator)

    Then call the rhythm-maker on arbitrary divisions::

        >>> divisions = [(5, 8), (5, 8), (5, 8)]
        >>> leaf_lists = maker(divisions)
        >>> leaves = sequencetools.flatten_sequence(leaf_lists)

    The resulting Abjad objects can be included in any score and the rhythm 
    maker can be reused::

        >>> measures = measuretools.make_measures_with_full_measure_spacer_skips(divisions)
        >>> staff = Staff(measures)
        >>> measures = measuretools.replace_contents_of_measures_in_expr(staff, leaves)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 5/8
                r4
                r4
                c'8
            }
            {
                c'2 ~
                c'8
            }
            {
                c'4
                r16.
                r16.
                r16.
                r16.
            }
        }

    Usage follows the two-step instantiate-then-call pattern shown here.

    Return rhythm  maker.
    '''

    ### PRIVATE METHODS ###

    def _make_middle_of_numeric_map_part(self, middle):
        if 0 < middle:
            return (middle, )
        else:
            return ()



from abjad.tools import componenttools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import leaftools


def split_container_at_index(component, i, fracture_spanners=False):
    r'''.. versionadded:: 1.1

    General component index split algorithm.
    Works on leaves, tuplets, measures, contexts and unqualified containers.
    Keyword controls spanner behavior at split time.
    Use containertools.split_container_at_index_and_fracture_crossing_spanners() 
    to fracture spanners.
    Use containertools.split_container_at_index_and_do_not_fracture_crossing_spanners() 
    to leave spanners unchanged.

    Example 1. Split container and do not fracture crossing spanners::

        >>> voice = Voice(Measure((3, 8), "c'8 c'8 c'8") * 2)
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(voice)
        >>> beam = beamtools.BeamSpanner(voice[:])

    ::

        >>> f(voice)
        \new Voice {
            {
                \time 3/8
                c'8 [
                d'8
                e'8
            }
            {
                f'8
                g'8
                a'8 ]
            }
        }

    ::

        >>> containertools.split_container_at_index(voice[1], 1, fracture_spanners=False)
        (Measure(1/8, [f'8]), Measure(2/8, [g'8, a'8]))

    ::

        >>> f(voice)
        \new Voice {
            {
                \time 3/8
                c'8 [
                d'8
                e'8
            }
            {
                \time 1/8
                f'8
            }
            {
                \time 2/8
                g'8
                a'8 ]
            }
        }

    Example 2. Split container and fracture crossing spanners::

        >>> voice = Voice(tuplettools.FixedDurationTuplet(Duration(2, 8), "c'8 c'8 c'8") * 2)
        >>> tuplet = voice[1]
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(voice)
        >>> beam = beamtools.BeamSpanner(voice[:])

    ::

        >>> f(voice)
        \new Voice {
            \times 2/3 {
                c'8 [
                d'8
                e'8
            }
            \times 2/3 {
                f'8
                g'8
                a'8 ]
            }
        }

    ::

        >>> left, right = containertools.split_container_at_index(
        ...         tuplet, 1, fracture_spanners=True)

    ::

        >>> f(voice)
        \new Voice {
            \times 2/3 {
                c'8 [
                d'8
                e'8
            }
            \times 2/3 {
                f'8 ]
            }
            \times 2/3 {
                g'8 [
                a'8 ]
            }
        }

    Leave spanners and leaves untouched.

    Resize resizable containers.

    Preserve container multiplier.

    Preserve time signature denominator.

    Return split parts.
    '''
    from abjad.tools import spannertools
    from abjad.tools import containertools
    from abjad.tools import contexttools
    from abjad.tools import measuretools
    from abjad.tools import tuplettools

    # convenience leaf index split definition
    if isinstance(component, leaftools.Leaf):
        #raise Exception # debug
        if i <= 0:
            if fracture_spanners:
                spannertools.fracture_spanners_attached_to_component(component, direction=Left)
            return None, component
        else:
            if fracture_spanners:
                spannertools.fracture_spanners_attached_to_component(component, direction=Right)
            return component, None

    # remember container multiplier, if any
    container_multiplier = getattr(component, 'implied_prolation', None)

    # partition music of input container
    left_music = component[:i]
    right_music = component[i:]

    # instantiate new left and right containers
    if isinstance(component, measuretools.Measure):
        time_signature_denominator = contexttools.get_effective_time_signature(component).denominator
        left_duration = sum([x.prolated_duration for x in left_music])
        left_pair = mathtools.NonreducedFraction(left_duration)
        left_pair = left_pair.with_multiple_of_denominator(time_signature_denominator)
        left_time_signature = contexttools.TimeSignatureMark(left_pair)
        left = component.__class__(left_time_signature, left_music)
        right_duration = sum([x.prolated_duration for x in right_music])
        right_pair = mathtools.NonreducedFraction(right_duration)
        right_pair = right_pair.with_multiple_of_denominator(time_signature_denominator)
        right_time_signature = contexttools.TimeSignatureMark(right_pair)
        right = component.__class__(right_time_signature, right_music)
    elif isinstance(component, tuplettools.FixedDurationTuplet):
        left = component.__class__(1, left_music)
        right = component.__class__(1, right_music)
        containertools.set_container_multiplier(left, container_multiplier)
        containertools.set_container_multiplier(right, container_multiplier)
    elif isinstance(component, tuplettools.Tuplet):
        left = component.__class__(container_multiplier, left_music)
        right = component.__class__(container_multiplier, right_music)
    else:
        left = component.__class__(left_music)
        right = component.__class__(right_music)
        containertools.set_container_multiplier(left, container_multiplier)
        containertools.set_container_multiplier(right, container_multiplier)

    # save left and right halves together for iteration
    halves = [left, right]
    nonempty_halves = [half for half in halves if len(half)]

    # give attached spanners to children
    spannertools.move_spanners_from_component_to_children_of_component(component)

    # incorporate left and right parents in score, if possible
    parent, start, stop = componenttools.get_parent_and_start_stop_indices_of_components([component])
    if parent is not None:
        # to avoid pychecker slice assignment error
        #parent._music[start:stop+1] = nonempty_halves
        parent._music.__setitem__(slice(start, stop + 1), nonempty_halves)
        for part in nonempty_halves:
            part._switch(parent)
    else:
        left._switch(None)
        right._switch(None)

    # fracture spanners, if requested
    if fracture_spanners:
        if len(halves) == 2:
            spannertools.fracture_spanners_attached_to_component(left, direction=Right)

    # return new left and right halves
    return left, right



from abjad.tools import durationtools


def calculate_mean_release_of_intervals(intervals):
    '''Return a Fraction of the average release offset of `intervals`.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    if not tree:
        return None

    return durationtools.Offset(sum([i.stop for i in tree])) / len(tree)



def get_lineage_of_component(component):
    r'''.. versionadded:: 2.9

    Get lineage of `component`::

        >>> staff = Staff(r"c'4 \times 2/3 { d'8 e'8 f'8 }")

    ::

        f(staff)
        \new Staff {
            c'4
            \times 2/3 {
                d'8
                e'8
                f'8
            }
        }

    ::

        componenttools.get_lineage_of_component(staff[1])
        [Staff{2}, Tuplet(2/3, [d'8, e'8, f'8]), Note("d'8"), Note("e'8"), Note("f'8")]

    Return list of parentage, component and descendents.
    '''
    from abjad.tools import componenttools

    # initialize result
    result = []

    # add parentage of component
    result.extend(reversed(componenttools.get_proper_parentage_of_component(component)))

    # add component
    result.append(component)

    # add descendents of component
    result.extend(componenttools.get_proper_descendents_of_component(component))

    # return result
    return result



from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript
import fnmatch
import os
import re


class ReplaceInFilesScript(DirectoryScript):
    '''Replace text in files recursively:

    ::

        bash$ ajv replace text -h
        usage: replace-in-files [-h] [--version] [--verbose] [-Y] [-R] [-W]
                                [-F PATTERN] [-D PATTERN]
                                path old new

        Replace text.

        positional arguments:
          path                  directory tree to be recursed over
          old                   old text
          new                   new text

        optional arguments:
          -h, --help            show this help message and exit
          --version             show program's version number and exit
          --verbose             print replacement info even when --force flag is set.
          -Y, --force           force "yes" to every replacement
          -R, --regex           treat "old" as a regular expression
          -W, --whole-words-only
                                match only whole words, similar to grep's "-w" flag
          -F PATTERN, --without-files PATTERN
                                Exclude files matching pattern(s)
          -D PATTERN, --without-dirs PATTERN
                                Exclude folders matching pattern(s)

    Multiple patterns for excluding files or folders can be specified by
    restating the `--without-files` or `--without-dirs` commands:

    ::

        bash$ ajv replace text . foo bar -F *.txt -F *.rst -F *.htm

    Return `ReplaceInFilesScript` instance.
    '''

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def alias(self):
        return 'text'

    @property
    def long_description(self):
        return None

    @property
    def scripting_group(self):
        return 'replace'

    @property
    def short_description(self):
        return 'Replace text.'

    @property
    def skipped_directories(self):
        return [
            '.svn',
            'build'
        ]

    @property
    def skipped_files(self):
        return [
            __file__,
            self.program_name,
            '*.doctree',
            '*.gif',
            '*.jpg',
            '*.jpeg',
            '*.ly',
            '*.nb',
            '*.pdf',
            '*.pickle',
            '*.pkl',
            '*.png',
            '*.ps',
            '*.pyc',
            '*.rtf',
            '*.tex',
            '*.txt',
        ]

    @property
    def version(self):
        return 1.0

    ### PRIVATE METHODS ###

    def _get_naive_search_callable(self, args):
        class NaiveSearch(object):
            def __init__(self, pattern):
                self.pattern = pattern
            def __call__(self, line, pos):
                index = line.find(self.pattern, pos)
                if 0 <= index:
                    return index, len(self.pattern)
                return -1, 0
        return NaiveSearch(args.old)

    def _get_regex_search_callable(self, args):
        class RegexSearch(object):
            def __init__(self, pattern, escaped=False, whole_words_only=False):
                try:
                    if escaped:
                        pattern = re.escape(pattern)
                    if whole_words_only:
                        pattern += r'\b'
                    self.pattern = re.compile(pattern)
                    self.whole_words_only = whole_words_only
                except:
                    raise ValueError("Can't compile {!r} as a regex pattern.".format(pattern))

            def __call__(self, line, pos):
                start, length = self._search(line, pos)
                if self.whole_words_only and 0 < start:
                    while start != -1 and (line[start-1].isalnum() or line[start-1] == '_'):
                        start, length = self._search(line, start + length)
                return start, length

            def _search(self, line, pos):
                match = self.pattern.search(line, pos)
                if match is None:
                    return -1, 0
                return match.start(), match.end() - match.start()

        return RegexSearch(args.old, escaped=not args.regex, whole_words_only=args.whole_words_only)

    def _process_file(self, args, path):

        changed_items = 0
        changed_lines = 0

        with open(path, 'r') as f:
            lines = f.read().split('\n')

        results = []
        for i, line in enumerate(lines):
            line, changes = self._process_line(line, i, path, args.old, args.new, args.force, args.verbose)
            results.append(line)
            if changes:
                changed_items += changes
                changed_lines += 1

        if results != lines:
            with open(path, 'w') as f:
                f.write('\n'.join(results))

        return changed_lines, changed_items

    def _process_line(self, line, line_number, filename, search, replacement, force, verbose):
        index, changes = 0, 0
        index, length = search(line, index)

        while 0 <= index:

            should_replace = False

            replaced_line = line[:index] + replacement + line[index+length:]
            carats = (' ' * index) + ('^' * length)

            if force:
                should_replace = True
                if verbose:
                    print ''
                    print '{}: {}'.format(filename, line_number)
                    print '-{}'.format(line)
                    print '+{}'.format(replaced_line)

            else:
                print ''
                print '{}: {}'.format(filename, line_number)
                print ''
                print '{}'.format(line)
                print '{}'.format(carats)
                print ''
                result = raw_input('Replace? [Y/n] > ').lower()
                while result not in ('', 'y', 'yes', 'n', 'no'):
                    result = raw_input('Replace? [Y/n] > ').lower()
                if result in ('', 'y', 'yes'):
                    should_replace = True

            if should_replace:
                index += length - (length - len(replacement))
                line = replaced_line
                changes += 1
            else:
                index += length

            index, length = search(line, index)

        return line, changes

    ### PUBLIC METHODS ###

    def process_args(self, args):

        print 'Replacing {!r} with {!r}...'.format(args.old, args.new)

        skipped_dirs_patterns = self.skipped_directories + args.without_dirs
        skipped_files_patterns = self.skipped_files + args.without_files

        if args.regex or (not args.regex and args.whole_words_only):
            args.old = self._get_regex_search_callable(args)
            index, length = args.old('', 0)
            if 0 <= index:
                raise ValueError('Regex pattern {!r} matches the empty string.'.format(args.old.pattern.pattern))
        else:
            args.old = self._get_naive_search_callable(args)

        changed_file_count = 0
        changed_line_count = 0
        changed_item_count = 0

        for root, dirs, files in os.walk(args.path):

            dirs_to_remove = []
            for dir in dirs:
                for pattern in skipped_dirs_patterns:
                    if fnmatch.fnmatch(dir, pattern):
                        dirs_to_remove.append(dir)
                        break
            for dir in dirs_to_remove:
                dirs.remove(dir)

            for file in sorted(files):
                valid = True
                for pattern in skipped_files_patterns:
                    if fnmatch.fnmatch(file, pattern):
                        valid = False
                        break
                if not valid:
                    continue

                changed_lines, changed_items = self._process_file(args, os.path.join(root, file))
                if changed_lines:
                    changed_file_count += 1
                    changed_line_count += changed_lines
                    changed_item_count += changed_items

        print ''
        print '\tReplaced {} instances over {} lines in {} files.'.format(
            changed_item_count, changed_line_count, changed_file_count)

    def setup_argument_parser(self, parser):

        parser.add_argument('path',
            type=self._validate_path,
            help='directory tree to be recursed over'
            )

        parser.add_argument('old',
            help='old text',
            )

        parser.add_argument('new',
            help='new text',
            )

        parser.add_argument('--verbose',
            action='store_true',
            help='print replacement info even when --force flag is set.',
            )

        parser.add_argument('-Y', '--force',
            action='store_true',
            help='force "yes" to every replacement'
            )

        parser.add_argument('-R', '--regex',
            action='store_true',
            help='treat "old" as a regular expression',
            )

        parser.add_argument('-W', '--whole-words-only',
            action='store_true',
            help='''match only whole words, similar to grep's "-w" flag''',
            )

        parser.add_argument('-F', '--without-files',
            action='append',
            default=[],
            help='Exclude files matching pattern(s)',
            metavar='PATTERN',
            )

        parser.add_argument('-D', '--without-dirs',
            action='append',
            default=[],
            help='Exclude folders matching pattern(s)',
            metavar='PATTERN',
            )



import abc
import copy
import fractions
from abjad.tools import durationtools
from abjad.tools import formattools
from abjad.tools import lilypondproxytools
from abjad.tools import mathtools
from abjad.tools import timespantools
from abjad.tools.abctools import AbjadObject


class Component(AbjadObject):

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta

    __slots__ = ('_duration', '_is_forbidden_to_update', '_marks_are_current',
        '_marks_for_which_component_functions_as_effective_context',
        '_marks_for_which_component_functions_as_start_component', 
        '_offset', '_offset_values_in_seconds_are_current', '_override', '_parent', 
        '_prolated_offset_values_are_current', '_set', '_spanners', 
        '_start_offset', '_start_offset_in_seconds', '_stop_offset', '_stop_offset_in_seconds', 
        'lilypond_file', )

    ### INITIALIZER ###

    def __init__(self):
        self._is_forbidden_to_update = False
        self._marks_are_current = False
        self._marks_for_which_component_functions_as_effective_context = list()
        self._marks_for_which_component_functions_as_start_component = list()
        self._offset_values_in_seconds_are_current = False
        self._parent = None
        self._prolated_offset_values_are_current = False
        self._spanners = set([])
        self._start_offset = None
        self._start_offset_in_seconds = None
        self._stop_offset = None
        self._stop_offset_in_seconds = None

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        return self._copy_with_marks_but_without_children_or_spanners()

    # TODO: make this the new definition of deepcopy
    #def __deepcopy__(self, memo):
    #    from abjad.tools import componenttools
    #    #return componenttools.copy_components_and_fracture_crossing_spanners([self])[0]
    #    return componenttools.copy_components_and_covered_spanners([self])[0]

    def __getnewargs__(self):
        return ()

    def __mul__(self, n):
        from abjad.tools import componenttools
        return componenttools.copy_components_and_remove_spanners([self], n)

    def __rmul__(self, n):
        return self * n

    ### PRIVATE PROPERTIES ###

    def _copy_with_children_and_marks_but_without_spanners(self):
        return self._copy_with_marks_but_without_children_or_spanners()

    def _copy_with_marks_but_without_children_or_spanners(self):
        from abjad.tools import marktools
        new = type(self)(*self.__getnewargs__())
        if getattr(self, '_override', None) is not None:
            new._override = copy.copy(self.override)
        if getattr(self, '_set', None) is not None:
            new._set = copy.copy(self.set)
        for mark in marktools.get_marks_attached_to_component(self):
            new_mark = copy.copy(mark)
            new_mark.attach(new)
        return new

    @property
    def _format_pieces(self):
        return self._format_component(pieces=True)

    @property
    def _id_string(self):
        lhs = self._class_name
        rhs = getattr(self, 'name', None) or id(self)
        return '{}-{!r}'.format(lhs, rhs)

    @property
    def _prolations(self):
        result = []
        parent = self.parent
        while parent is not None:
            result.append(getattr(parent, 'implied_prolation', fractions.Fraction(1)))
            parent = parent.parent
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def descendants(self):
        '''Read-only reference to component descendants score selection.'''
        from abjad.tools import componenttools
        return componenttools.Descendants(self)

    @abc.abstractproperty
    def duration_in_seconds(self):
        pass

    @property
    def lilypond_format(self):
        self._update_marks_of_entire_score_tree_if_necessary()
        return self._format_component()

    @property
    def lineage(self):
        '''Read-only reference to component lineage score selection.'''
        from abjad.tools import componenttools
        return componenttools.Lineage(self)

    @property
    def override(self):
        '''Read-only reference to LilyPond grob override component plug-in.
        '''
        if not hasattr(self, '_override'):
            self._override = lilypondproxytools.LilyPondGrobOverrideComponentPlugIn()
        return self._override

    @property
    def parent(self):
        return self._parent

    @property
    def parentage(self):
        '''Read-only reference to component parentage score selection.'''
        from abjad.tools import componenttools
        return componenttools.Parentage(self)

    @property
    def prolated_duration(self):
        return self.prolation * self.preprolated_duration

    @property
    def prolation(self):
        products = mathtools.cumulative_products([fractions.Fraction(1)] + self._prolations)
        return products[-1]

    @property
    def set(self):
        '''Read-only reference LilyPond context setting component plug-in.
        '''
        if not hasattr(self, '_set'):
            self._set = lilypondproxytools.LilyPondContextSettingComponentPlugIn()
        return self._set

    @property
    def spanners(self):
        '''Read-only reference to unordered set of spanners attached to component.
        '''
        return set(self._spanners)

    @property
    def start_offset(self):
        '''Read-only start offset of component.
        '''
        self._update_prolated_offset_values_of_entire_score_tree_if_necessary()
        return self._start_offset

    @property
    def start_offset_in_seconds(self):
        '''Read-only start offset of comonent in seconds.
        '''
        self._update_marks_of_entire_score_tree_if_necessary()
        if self._start_offset_in_seconds is None:
            raise MissingTempoError
        return self._start_offset_in_seconds

    @property
    def stop_offset(self):
        '''Read-only stop offset of component.
        '''
        return self.start_offset + self.prolated_duration

    @property
    def stop_offset_in_seconds(self):
        '''Read-only stop offset of component in seconds.
        '''
        return self.start_offset_in_seconds + self.duration_in_seconds

    @property
    def timespan(self):
        '''Read-only timespan of component.
        '''
        return timespantools.Timespan(start_offset=self.start_offset, stop_offset=self.stop_offset)

    @property
    def timespan_in_seconds(self):
        '''Read-only timespan of component in seconds.
        '''
        return timespantools.Timespan(
            start_offset=self.start_offset_in_seconds, stop_offset=self.stop_offset_in_seconds)

    ### PRIVATE METHODS ###

    def _cut(self):
        '''Component loses pointer to parent and parent loses pointer to component.
        Not composer-safe.
        '''
        if self.parent is not None:
            index = self.parent.index(self)
            self.parent._music.pop(index)
        self._ignore()

    def _format_after_slot(self, format_contributions):
        pass

    def _format_before_slot(self, format_contributions):
        pass

    def _format_close_brackets_slot(self, format_contributions):
        pass

    def _format_closing_slot(self, format_contributions):
        pass

    def _format_component(self, pieces=False):
        result = []
        format_contributions = formattools.get_all_format_contributions(self)
        result.extend(self._format_before_slot(format_contributions))
        result.extend(self._format_open_brackets_slot(format_contributions))
        result.extend(self._format_opening_slot(format_contributions))
        result.extend(self._format_contents_slot(format_contributions))
        result.extend(self._format_closing_slot(format_contributions))
        result.extend(self._format_close_brackets_slot(format_contributions))
        result.extend(self._format_after_slot(format_contributions))
        contributions = []
        for contributor, contribution in result:
            contributions.extend(contribution)
        if pieces:
            return contributions
        else:
            return '\n'.join(contributions)

    def _format_contents_slot(self, format_contributions):
        pass

    def _format_open_brackets_slot(self, format_contributions):
        pass

    def _format_opening_slot(self, format_contributions):
        pass

    def _get_format_contributions_for_slot(self, n, format_contributions=None):
        if format_contributions is None:
            format_contributions = formattools.get_all_format_contributions(self)
        result = []
        slots = ('before', 'open_brackets', 'opening',
            'contents', 'closing', 'close_brackets', 'after')
        if isinstance(n, str):
            n = n.replace(' ', '_')
        elif isinstance(n, int):
            n = slots[n-1]
        attr = getattr(self, '_format_{}_slot'.format(n))
        for source, contributions in attr(format_contributions):
            result.extend(contributions)
        return result

    # TODO: change name to something self-documenting
    def _ignore(self):
        '''Component loses pointer to parent but parent preserves pointer to component.
        Not composer-safe.
        '''
        self._mark_entire_score_tree_for_later_update('prolated')
        self._parent = None

    def _initialize_keyword_values(self, **kwargs):
        for key, value in kwargs.iteritems():
            self._set_keyword_value(key, value)

    def _set_keyword_value(self, key, value):
        attribute_chain = key.split('__')
        plug_in_name = attribute_chain[0]
        names = attribute_chain[1:]
        if plug_in_name == 'duration':
            attribute_name = names[0]
            command = 'self.%s.%s = %r' % (plug_in_name, attribute_name, value)
            print command
            if 'multiplier' not in command:
                exec(command)
        elif plug_in_name == 'override':
            if len(names) == 2:
                grob_name, attribute_name = names
                exec('self.override.%s.%s = %r' % (grob_name, attribute_name, value))
            elif len(names) == 3:
                context_name, grob_name, attribute_name = names
                exec('self.override.%s.%s.%s = %r' % (
                    context_name, grob_name, attribute_name, value))
            else:
                raise ValueError
        elif plug_in_name == 'set':
            if len(names) == 1:
                setting_name = names[0]
                exec('self.set.%s = %r' % (setting_name, value))
            elif len(names) == 2:
                context_name, setting_name = names
                exec('self.set.%s.%s = %r' % (context_name, setting_name, value))
            else:
                raise ValueError
        else:
            raise ValueError('\n\t: Unknown keyword argument plug-in name: "%s".' % plug_in_name)

    # TODO: rename to something self-documenting
    def _switch(self, new_parent):
        '''Component loses pointer to parent and parent loses pointer to component.
        Then assign component to new parent.
        '''
        from abjad.tools import componenttools

        name_dictionary = {}
        if hasattr(self, '_named_children'):
            for name, children in self._named_children.iteritems():
                name_dictionary[name] = copy.copy(children)
        if hasattr(self, 'name') and self.name is not None:
            if self.name not in name_dictionary:
                name_dictionary[self.name] = []
            name_dictionary[self.name].append(self)

        if self._parent is not None and name_dictionary:
            parentage = componenttools.get_proper_parentage_of_component(self)
            for parent in parentage:
                named_children = parent._named_children
                for name in name_dictionary:
                    for component in name_dictionary[name]:
                        named_children[name].remove(component)
                    if not named_children[name]:
                        del named_children[name]

        self._cut()
        self._parent = new_parent

        if new_parent is not None and name_dictionary:
            parentage = componenttools.get_proper_parentage_of_component(self)
            for parent in parentage:
                named_children = parent._named_children
                for name in name_dictionary:
                    if name in named_children:
                        named_children[name].extend(name_dictionary[name])
                    else:
                        named_children[name] = copy.copy(name_dictionary[name])

        self._mark_entire_score_tree_for_later_update('prolated')

    ### MANGLED METHODS ###

    def _update_leaf_indices_and_measure_numbers_in_score_tree(self):
        '''Called only when updating prolated offset of score components.
        No separate state flags for leaf indices or measure numbers.
        '''
        from abjad.tools import componenttools
        from abjad.tools import contexttools
        from abjad.tools import iterationtools
        from abjad.tools import leaftools
        from abjad.tools import measuretools
        from abjad.tools.contexttools.Context import Context
        score_root = self.parentage.root
        if isinstance(score_root, Context):
            for context in iterationtools.iterate_contexts_in_expr(score_root):
                for leaf_index, leaf in enumerate(iterationtools.iterate_leaves_in_expr(context)):
                    leaf._leaf_index = leaf_index
                for measure_index, measure in enumerate(
                    iterationtools.iterate_measures_in_expr(context)):
                    measure_number = measure_index + 1
                    measure._measure_number = measure_number
        else:
            for leaf_index, leaf in enumerate(iterationtools.iterate_leaves_in_expr(score_root)):
                leaf._leaf_index = leaf_index
            for measure_index, measure in enumerate(
                iterationtools.iterate_measures_in_expr(score_root)):
                measure_number = measure_index + 1
                measure._measure_number = measure_number

    def _update_marks_of_entire_score_tree(self):
        '''Updating marks does not cause prolated offset values to update.
        On the other hand, getting effective mark causes prolated offset values
        to update when at least one mark of appropriate type attaches to score.
        '''
        components = self._iterate_score_components_depth_first()
        for component in components:
            for mark in component._marks_for_which_component_functions_as_start_component:
                if hasattr(mark, '_update_effective_context'):
                    mark._update_effective_context()
            component._marks_are_current = True

    def _update_offset_values_in_seconds_of_entire_score_tree(self):
        from abjad.tools import offsettools
        components = self._iterate_score_components_depth_first()
        for component in components:
            offsettools.update_offset_values_of_component_in_seconds(component)
            component._offset_values_in_seconds_are_current = True

    def _update_prolated_offset_values_of_entire_score_tree(self):
        '''Updating prolated offset values does NOT update marks.
        Updating prolated offset values does NOT update offset values in seconds.
        '''
        from abjad.tools import offsettools
        components = self._iterate_score_components_depth_first()
        for component in components:
            offsettools.update_offset_values_of_component(component)
            component._prolated_offset_values_are_current = True

    ### PRIVATE UPDATE METHODS ###

    def _allow_component_update(self):
        self._is_forbidden_to_update = False

    def _forbid_component_update(self):
        self._is_forbidden_to_update = True

    def _get_score_tree_state_flags(self):
        from abjad.tools import componenttools
        prolated_offset_values_are_current = True
        marks_are_current = True
        offset_values_in_seconds_are_current = True
        for component in componenttools.get_improper_parentage_of_component(self):
            if prolated_offset_values_are_current:
                if not component._prolated_offset_values_are_current:
                    prolated_offset_values_are_current = False
            if marks_are_current:
                if not component._marks_are_current:
                    marks_are_current = False
            if offset_values_in_seconds_are_current:
                if not component._offset_values_in_seconds_are_current:
                    offset_values_in_seconds_are_current = False
        return (prolated_offset_values_are_current, marks_are_current,
            offset_values_in_seconds_are_current)

    def _iterate_score_components_depth_first(self):
        from abjad.tools import componenttools
        from abjad.tools import iterationtools
        kwargs = {'capped': True, 'unique': True, 'forbid': None, 'direction': 'left'}
        components = iterationtools.iterate_components_depth_first(self.parentage.root, **kwargs)
        return components

    def _mark_entire_score_tree_for_later_update(self, value):
        '''Call immediately AFTER MODIFYING score tree.
        Only dynamic measures mark time signature for udpate.
        '''
        from abjad.tools import componenttools
        assert value in ('prolated', 'marks', 'seconds')
        for component in componenttools.get_improper_parentage_of_component(self):
            if value == 'prolated':
                component._prolated_offset_values_are_current = False
            elif value == 'marks':
                component._marks_are_current = False
            elif value == 'seconds':
                component._offset_values_in_seconds_are_current = False
            else:
                raise ValueError('unknown value: "%s"' % value)
            if hasattr(component, '_time_signature_is_current'):
                component._time_signature_is_current = False

    def _update_marks_of_entire_score_tree_if_necessary(self):
        '''Call immediately BEFORE READING effective mark.
        '''
        if self._is_forbidden_to_update:
            return
        state_flags = self._get_score_tree_state_flags()
        marks_are_current = state_flags[1]
        if not marks_are_current:
            self._update_marks_of_entire_score_tree()
            self._update_offset_values_in_seconds_of_entire_score_tree()

    def _update_prolated_offset_values_of_entire_score_tree_if_necessary(self):
        if self._is_forbidden_to_update:
            return
        state_flags = self._get_score_tree_state_flags()
        prolated_offset_values_are_current = state_flags[0]
        if not prolated_offset_values_are_current:
            self._update_prolated_offset_values_of_entire_score_tree()
            self._update_leaf_indices_and_measure_numbers_in_score_tree()



from abjad import *
from abjad.tools.quantizationtools import *
import py


def test_QGrid___call___01():
    
    q_grid = QGrid()
    a = QEventProxy(SilentQEvent(0,        ['A']), 0)
    b = QEventProxy(SilentQEvent((1, 20),  ['B']), (1, 20))
    c = QEventProxy(SilentQEvent((9, 20),  ['C']), (9, 20))
    d = QEventProxy(SilentQEvent((1, 2),   ['D']), (1, 2))
    e = QEventProxy(SilentQEvent((11, 20), ['E']), (11, 20))
    f = QEventProxy(SilentQEvent((19, 20), ['F']), (19, 20))
    g = QEventProxy(SilentQEvent(1,        ['G']), 1)
    q_grid.fit_q_events([a, b, c, d, e, f, g])
    result = q_grid((1, 4))

    assert isinstance(result, list) and len(result) == 1
    assert result[0].lilypond_format == "c'4"

    annotation = marktools.get_annotation_attached_to_component(result[0])

    assert isinstance(annotation.value, tuple) and len(annotation.value) == 4
    assert annotation.value[0].attachments == ('A',)
    assert annotation.value[1].attachments == ('B',)
    assert annotation.value[2].attachments == ('C',)
    assert annotation.value[3].attachments == ('D',)


def test_QGrid___call___02():

    q_grid = QGrid()
    q_grid.subdivide_leaves([(0, (1, 1, 1))])
    q_grid.subdivide_leaves([(1, (1, 1))])
    q_grid.subdivide_leaves([(-2, (1, 1, 1))])
    a = QEventProxy(SilentQEvent(0,        ['A']), 0)
    b = QEventProxy(SilentQEvent((1, 20),  ['B']), (1, 20))
    c = QEventProxy(SilentQEvent((9, 20),  ['C']), (9, 20))
    d = QEventProxy(SilentQEvent((1, 2),   ['D']), (1, 2))
    e = QEventProxy(SilentQEvent((11, 20), ['E']), (11, 20))
    f = QEventProxy(SilentQEvent((19, 20), ['F']), (19, 20))
    g = QEventProxy(SilentQEvent(1,        ['G']), 1)
    q_grid.fit_q_events([a, b, c, d, e, f, g])
    result = q_grid((1, 4))

    assert isinstance(result, list) and len(result) == 1
    assert result[0].lilypond_format == "\\times 2/3 {\n\tc'8\n\tc'16\n\tc'16\n\t\\times 2/3 {\n\t\tc'16\n\t\tc'16\n\t\tc'16\n\t}\n}"

    annotation = marktools.get_annotations_attached_to_component(result[0].leaves[0])[0]
    assert isinstance(annotation.value, tuple) and len(annotation.value) == 2
    assert annotation.value[0].attachments == ('A',)
    assert annotation.value[1].attachments == ('B',)
    
    assert not marktools.get_annotations_attached_to_component(result[0].leaves[1])

    annotation = marktools.get_annotation_attached_to_component(result[0].leaves[2])
    assert isinstance(annotation.value, tuple) and len(annotation.value) == 3
    assert annotation.value[0].attachments == ('C',)
    assert annotation.value[1].attachments == ('D',)
    assert annotation.value[2].attachments == ('E',)
    
    assert not marktools.get_annotations_attached_to_component(result[0].leaves[3])
    assert not marktools.get_annotations_attached_to_component(result[0].leaves[4])
    assert not marktools.get_annotations_attached_to_component(result[0].leaves[5])


def test_QGrid___call___03():
    '''Non-binary works too.'''

    q_grid = QGrid()
    q_grid.subdivide_leaves([(0, (1, 1))])

    a = QEventProxy(SilentQEvent(0,        ['A']), 0)
    b = QEventProxy(SilentQEvent((1, 20),  ['B']), (1, 20))
    c = QEventProxy(SilentQEvent((9, 20),  ['C']), (9, 20))
    d = QEventProxy(SilentQEvent((1, 2),   ['D']), (1, 2))
    e = QEventProxy(SilentQEvent((11, 20), ['E']), (11, 20))
    f = QEventProxy(SilentQEvent((19, 20), ['F']), (19, 20))
    g = QEventProxy(SilentQEvent(1,        ['G']), 1)

    q_grid.fit_q_events([a, b, c, d, e, f, g])

    result = q_grid((1, 3))

    assert isinstance(result, list) and len(result) == 1
    assert result[0].lilypond_format == "\\times 2/3 {\n\tc'4\n\tc'4\n}"





import abc

from abjad.tools.pitchtools.DiatonicObject import DiatonicObject
from abjad.tools.pitchtools.PitchObject import PitchObject


class DiatonicPitchObject(PitchObject, DiatonicObject):
    '''.. versionadded:: 2.0

    Diatonic pitch base class.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta

    __slots__ = ()


    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __abs__(self):
        return self._number

    def __float__(self):
        return float(self._number)

    def __int__(self):
        return self._number



from abjad import *


def test_GroupedRhythmicStavesScoreTemplate___call___01():

    template = scoretemplatetools.GroupedRhythmicStavesScoreTemplate(staff_count=4)
    score = template()

    assert score.lilypond_format == '\\context Score = "Grouped Rhythmic Staves Score" <<\n\t\\context StaffGroup = "Grouped Rhythmic Staves Staff Group" <<\n\t\t\\context RhythmicStaff = "Staff 1" {\n\t\t\t\\context Voice = "Voice 1" {\n\t\t\t}\n\t\t}\n\t\t\\context RhythmicStaff = "Staff 2" {\n\t\t\t\\context Voice = "Voice 2" {\n\t\t\t}\n\t\t}\n\t\t\\context RhythmicStaff = "Staff 3" {\n\t\t\t\\context Voice = "Voice 3" {\n\t\t\t}\n\t\t}\n\t\t\\context RhythmicStaff = "Staff 4" {\n\t\t\t\\context Voice = "Voice 4" {\n\t\t\t}\n\t\t}\n\t>>\n>>'


def test_GroupedRhythmicStavesScoreTemplate___call___02():

    template = scoretemplatetools.GroupedRhythmicStavesScoreTemplate(staff_count=[2, 1, 2])
    score = template()

    assert score.lilypond_format == '\\context Score = "Grouped Rhythmic Staves Score" <<\n\t\\context StaffGroup = "Grouped Rhythmic Staves Staff Group" <<\n\t\t\\context RhythmicStaff = "Staff 1" <<\n\t\t\t\\context Voice = "Voice 1-1" {\n\t\t\t}\n\t\t\t\\context Voice = "Voice 1-2" {\n\t\t\t}\n\t\t>>\n\t\t\\context RhythmicStaff = "Staff 2" {\n\t\t\t\\context Voice = "Voice 2" {\n\t\t\t}\n\t\t}\n\t\t\\context RhythmicStaff = "Staff 3" <<\n\t\t\t\\context Voice = "Voice 3-1" {\n\t\t\t}\n\t\t\t\\context Voice = "Voice 3-2" {\n\t\t\t}\n\t\t>>\n\t>>\n>>'



import types
from abjad.tools.documentationtools.Documenter import Documenter


class FunctionDocumenter(Documenter):
    '''FunctionDocumenter generates an ReST entry for a given function:

    ::

        >>> from abjad.tools.documentationtools import *

    ::

        >>> from abjad.tools.notetools import make_notes
        >>> documenter = FunctionDocumenter(make_notes)
        >>> print documenter()
        notetools.make_notes
        ====================
        <BLANKLINE>
        .. autofunction:: abjad.tools.notetools.make_notes.make_notes
           :noindex:

    Returns ``FunctionDocumenter``` instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_object', '_prefix')

    ### INITIALIZER ###

    def __init__(self, obj, prefix='abjad.tools.'):
        assert isinstance(obj, types.FunctionType)
        if obj.__name__ == 'wrapper':
            obj = obj.func_closure[1].cell_contents
        Documenter.__init__(self, obj, prefix)

    ### SPECIAL METHODS ###

    def __call__(self):
        '''Generate documentation.

        Returns string.
        '''
        from abjad.tools import documentationtools

        document = documentationtools.ReSTDocument()
        heading = documentationtools.ReSTHeading(
            level=2,
            text=self._shrink_module_name(self.object.__module__)
            )
        autodoc = documentationtools.ReSTAutodocDirective(
            argument=self.module_name,
            directive='autofunction',
            options={'noindex': True},
            )
        document.extend([heading, autodoc])

        return document.rest_format



from abjad import *


def test_componenttools_partition_components_by_durations_not_greater_than_01():

    t = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 4)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)
    contexttools.TempoMark(Duration(1, 4), 60, target_context=Staff)(t)

    r'''
    \new Staff {
            \time 2/8
            \tempo 4=60
            c'8
            d'8
            \time 2/8
            e'8
            f'8
            \time 2/8
            g'8
            a'8
            \time 2/8
            b'8
            c''8
            %% tempo 4=60 ends here
    }
    '''

    parts = componenttools.partition_components_by_durations_not_greater_than(
        t.leaves, [0.75], cyclic=True, in_seconds=True, overhang=False)

    "[[Note(c', 8)], [Note(d', 8)], [Note(e', 8)], [Note(f', 8)], [Note(g', 8)], [Note(a', 8)], [Note(b', 8)]]"

    assert len(parts) == 7
    assert parts[0] == list(t.leaves[:1])
    assert parts[1] == list(t.leaves[1:2])
    assert parts[2] == list(t.leaves[2:3])
    assert parts[3] == list(t.leaves[3:4])
    assert parts[4] == list(t.leaves[4:5])
    assert parts[5] == list(t.leaves[5:6])
    assert parts[6] == list(t.leaves[6:7])


def test_componenttools_partition_components_by_durations_not_greater_than_02():

    t = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 4)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)

    r'''
    \new Staff {
            \time 2/8
            c'8
            d'8
            \time 2/8
            e'8
            f'8
            \time 2/8
            g'8
            a'8
            \time 2/8
            b'8
            c''8
    }
    '''

    parts = componenttools.partition_components_by_durations_not_greater_than(
        t.leaves, [Duration(3, 16)], cyclic=True, in_seconds=False, overhang=False)

    "[[Note(c', 8)], [Note(d', 8)], [Note(e', 8)], [Note(f', 8)], "
    "[Note(g', 8)], [Note(a', 8)], [Note(b', 8)]]"

    assert len(parts) == 7
    assert parts[0] == list(t.leaves[:1])
    assert parts[1] == list(t.leaves[1:2])
    assert parts[2] == list(t.leaves[2:3])
    assert parts[3] == list(t.leaves[3:4])
    assert parts[4] == list(t.leaves[4:5])
    assert parts[5] == list(t.leaves[5:6])
    assert parts[6] == list(t.leaves[6:7])


def test_componenttools_partition_components_by_durations_not_greater_than_03():

    t = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 4)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)
    contexttools.TempoMark(Duration(1, 4), 60, target_context = Staff)(t)

    r'''
    \new Staff {
            \time 2/8
            \tempo 4=60
            c'8
            d'8
            \time 2/8
            e'8
            f'8
            \time 2/8
            g'8
            a'8
            \time 2/8
            b'8
            c''8
            %% tempo 4=60 ends here
    }
    '''

    parts = componenttools.partition_components_by_durations_not_greater_than(
            t.leaves, [0.75], cyclic=False, in_seconds=True, overhang=False)

    "[[Note(c', 8)]]"

    assert len(parts) == 1
    assert parts[0] == list(t.leaves[:1])


def test_componenttools_partition_components_by_durations_not_greater_than_04():

    t = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 4)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)

    r'''
    \new Staff {
            \time 2/8
            c'8
            d'8
            \time 2/8
            e'8
            f'8
            \time 2/8
            g'8
            a'8
            \time 2/8
            b'8
            c''8
    }
    '''

    parts = componenttools.partition_components_by_durations_not_greater_than(
        t.leaves, [Duration(3, 16)], cyclic=False, in_seconds=False, overhang=False)

    "[[Note(c', 8)]]"

    assert len(parts) == 1
    assert parts[0] == list(t.leaves[:1])



from abjad.tools.quantizationtools.JobHandler import JobHandler


class SerialJobHandler(JobHandler):
    '''Processes ``QuantizationJob`` instances sequentially.
    '''

    ### SPECIAL METHODS ###

    def __call__(self, jobs):
        for job in jobs:
            job()
        return jobs        



def detach_marks_attached_to_component(component):
    r'''.. versionadded:: 2.0

    Detach marks attached to `component`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> slur = spannertools.SlurSpanner(staff.leaves)
        >>> marktools.Articulation('^')(staff[0])
        Articulation('^')(c'8)
        >>> marktools.LilyPondComment('comment 1')(staff[0])
        LilyPondComment('comment 1')(c'8)
        >>> marktools.LilyPondCommandMark('slurUp')(staff[0])
        LilyPondCommandMark('slurUp')(c'8)

    ::

        >>> f(staff)
        \new Staff {
            % comment 1
            \slurUp
            c'8 -\marcato (
            d'8
            e'8
            f'8 )
        }

    ::

        >>> for mark in marktools.get_marks_attached_to_component(staff[0]):
        ...     mark
        ...
        Articulation('^')(c'8)
        LilyPondComment('comment 1')(c'8)
        LilyPondCommandMark('slurUp')(c'8)

    ::

        >>> for mark in marktools.detach_marks_attached_to_component(staff[0]):
        ...     mark
        ...
        Articulation('^')
        LilyPondComment('comment 1')
        LilyPondCommandMark('slurUp')

    ::

        >>> marktools.get_marks_attached_to_component(staff[0])
        ()

    Return tuple or zero or more marks detached.
    '''
    from abjad.tools import marktools

    marks = []
    for mark in marktools.get_marks_attached_to_component(component):
        mark.detach()
        marks.append(mark)

    return tuple(marks)



from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools.Flute import Flute


class Piccolo(Flute):
    r'''.. versionadded:: 2.0

    Abjad model of the piccolo::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> instrumenttools.Piccolo()(staff)
        Piccolo()(Staff{4})

    ::

        >>> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { Piccolo }
            \set Staff.shortInstrumentName = \markup { Picc. }
            c'8
            d'8
            e'8
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    The piccolo targets staff context by default.
    '''

    def __init__(self, **kwargs):
        Flute.__init__(self, **kwargs)
        self._default_instrument_name = 'piccolo'
        self._default_short_instrument_name = 'picc.'
        self._is_primary_instrument = False
        self.sounding_pitch_of_fingered_middle_c = pitchtools.NamedChromaticPitch("c''")
        self._traditional_pitch_range = pitchtools.PitchRange(14, 48)



def get_improper_descendents_of_component(component):
    r'''.. versionadded:: 2.9

    Get improper descendents of `component`::

        >>> staff = Staff(r"c'4 \times 2/3 { d'8 e'8 f'8 }")

    ::

        >>> f(staff)
        \new Staff {
            c'4
            \times 2/3 {
                d'8
                e'8
                f'8
            }
        }

    ::

        >>> for x in componenttools.get_improper_descendents_of_component(staff):
        ...     x
        ...
        Staff{2}
        Note("c'4")
        Tuplet(2/3, [d'8, e'8, f'8])
        Note("d'8")
        Note("e'8")
        Note("f'8")

    Function returns exactly the same components as 
    ``iterationtools.iterate_components_in_expr()``.

    Return list of `component` together with proper descendents of `component`.
    '''
    from abjad.tools import iterationtools

    # return list of component and all descendents of component
    return list(iterationtools.iterate_components_in_expr(component))



'''Package: abjad.tools.tietools

    TBD. 

'''

from abjad.tools import importtools

importtools.import_structured_package(__path__[0], globals(), package_root_name='abjad')

_documentation_section = 'core'



import os
import shutil
import subprocess
import tempfile


def compare_images(image_one, image_two):
    '''Compare `image_one` against `image_two` using ImageMagick's `compare`
    commandline tool.

    Return `True` if images are the same, otherwise `False`.
    '''

    assert os.path.exists(image_one)
    assert os.path.exists(image_two)

    tempdir = tempfile.mkdtemp()
    comparison = os.path.join(tempdir, 'comparison.png')

    command = 'compare -metric ae {} {} {}'.format(
        image_one, image_two, comparison)
    process = subprocess.Popen(command.split(),
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE
        )

    stderr = process.stderr.read()
    result = True
    if stderr.startswith('compare: image widths or heights differ'):
        result = False
    else:
        result = int(stderr.split('\n')[0].split()[0]) is 0

    shutil.rmtree(tempdir)
    return result



from abjad.tools.formattools.get_all_mark_format_contributions import get_all_mark_format_contributions
from abjad.tools.formattools.get_context_setting_format_contributions import get_context_setting_format_contributions
from abjad.tools.formattools.get_grob_override_format_contributions import get_grob_override_format_contributions
from abjad.tools.formattools.get_grob_revert_format_contributions import get_grob_revert_format_contributions
from abjad.tools.formattools.get_spanner_format_contributions import get_spanner_format_contributions


def get_all_format_contributions(component):
    '''Get all format contributions for `component` as a nested dictionary structure.

    Return dict.
    '''

    result = get_all_mark_format_contributions(component)

    for slot, contributions in get_spanner_format_contributions(component).iteritems():
        if slot not in result:
            result[slot] = { }
        result[slot]['spanners'] = contributions

    settings = get_context_setting_format_contributions(component)[1]
    if settings:
        result['context settings'] = settings

    overrides = get_grob_override_format_contributions(component)[1]
    if overrides:
        result['grob overrides'] = overrides
    
    reverts = get_grob_revert_format_contributions(component)[1]
    if reverts:
        result['grob reverts'] = reverts

    return result



def sort_named_chromatic_pitch_carriers_in_expr(pitch_carriers):
    '''.. versionadded:: 2.0

    List named chromatic pitch carriers in `expr` sorted by numbered chromatic pitch-class::

        >>> chord = Chord([9, 11, 12, 14, 16], (1, 4))
        >>> notes = chordtools.arpeggiate_chord(chord)

    ::

        >>> pitchtools.sort_named_chromatic_pitch_carriers_in_expr(notes)
        [Note("c''4"), Note("d''4"), Note("e''4"), Note("a'4"), Note("b'4")]

    The elements in `pitch_carriers` are not changed in any way.

    Return list.
    '''
    from abjad.tools import pitchtools

    result = list(pitch_carriers[:])
    result.sort(lambda x, y: cmp(
            abs(pitchtools.list_named_chromatic_pitches_in_expr(x)[0].numbered_chromatic_pitch_class),
            abs(pitchtools.list_named_chromatic_pitches_in_expr(y)[0].numbered_chromatic_pitch_class)))

    return result



def make_measures_with_full_measure_spacer_skips(time_signatures):
    r'''.. versionadded:: 1.1

    Make measures with full-measure spacer skips from `time_signatures`::

        >>> measures = measuretools.make_measures_with_full_measure_spacer_skips(
        ...     [(1, 8), (5, 16), (5, 16)])

    ::

        >>> staff = Staff(measures)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 1/8
                s1 * 1/8
            }
            {
                \time 5/16
                s1 * 5/16
            }
            {
                s1 * 5/16
            }
        }

    Return list of rigid measures.

    .. versionchanged:: 2.0
        renamed ``measuretools.make()`` to
        ``measuretools.make_measures_with_full_measure_spacer_skips()``.
    '''
    from abjad.tools import contexttools
    from abjad.tools import measuretools

    # check input
    time_signatures = [contexttools.TimeSignatureMark(x) for x in time_signatures]

    # make measures
    measures = [measuretools.Measure(x, []) for x in time_signatures]
    measuretools.fill_measures_in_expr_with_full_measure_spacer_skips(measures)

    # return measures
    return measures



from abjad.tools import durationtools
from abjad.tools import mathtools


def set_denominator_of_tuplets_in_expr_to_at_least(expr, n):
    r'''.. versionadded:: 2.0

    Set denominator of tuplets in `expr` to at least `n`::

        >>> tuplet = Tuplet(Fraction(3, 5), "c'4 d'8 e'8 f'4 g'2")

    ::

        >>> f(tuplet)
        \fraction \times 3/5 {
            c'4
            d'8
            e'8
            f'4
            g'2
        }

    ::

        >>> tuplettools.set_denominator_of_tuplets_in_expr_to_at_least(tuplet, 8)

    ::

        >>> f(tuplet)
        \fraction \times 6/10 {
            c'4
            d'8
            e'8
            f'4
            g'2
        }

    Return none.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import tuplettools

    assert mathtools.is_nonnegative_integer_power_of_two(n)
    for tuplet in iterationtools.iterate_tuplets_in_expr(expr):
        tuplet.force_fraction = True
        durations = [tuplet.contents_duration, tuplet.preprolated_duration, (1, n)]
        duration_pairs = durationtools.durations_to_nonreduced_fractions_with_common_denominator(
            durations)
        tuplet.preferred_denominator = duration_pairs[1].numerator



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.demos.desordre.make_desordre_cell import make_desordre_cell


def make_desordre_measure(pitches):
    '''Constructs a measure composed of *Désordre cells*.
    `pitches` is a list of lists of number (e.g., [[1, 2, 3], [2, 3, 4]])
    The function returns a DynamicMeasure.
    '''
    measure = measuretools.DynamicMeasure([ ])
    for sequence in pitches:
        measure.append(make_desordre_cell(sequence))

    # make denominator 8
    if contexttools.get_effective_time_signature(measure).denominator == 1:
        measure.denominator = 8

    return measure



# -*- encoding: utf-8 -*-
from abjad.tools import contexttools
from abjad.tools import instrumenttools
from abjad.tools import scoretools
from abjad.tools import stafftools
from abjad.tools import voicetools
from abjad.tools import scoretemplatetools


class PartCantusScoreTemplate(scoretemplatetools.ScoreTemplate):

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __call__(self):

        # make bell voice and staff
        bell_voice = voicetools.Voice(name='Bell Voice')
        bell_staff = stafftools.Staff([bell_voice], name='Bell Staff')
        contexttools.ClefMark('treble')(bell_staff)
        contexttools.InstrumentMark('Campana in La', 'Camp.')(bell_staff)
        contexttools.TempoMark((1, 4), (112, 120))(bell_staff)
        contexttools.TimeSignatureMark((6, 4))(bell_staff)

        # make first violin voice and staff
        first_violin_voice = voicetools.Voice(name='First Violin Voice')
        first_violin_staff = stafftools.Staff([first_violin_voice], name='First Violin Staff')
        contexttools.ClefMark('treble')(first_violin_staff)
        instrumenttools.Violin(
            instrument_name_markup='Violin I', 
            short_instrument_name_markup='Vl. I'
            )(first_violin_staff)

        # make second violin voice and staff
        second_violin_voice = voicetools.Voice(name='Second Violin Voice')
        second_violin_staff = stafftools.Staff([second_violin_voice], name='Second Violin Staff')
        contexttools.ClefMark('treble')(second_violin_staff)
        instrumenttools.Violin(
            instrument_name_markup='Violin II', 
            short_instrument_name_markup='Vl. II'
            )(second_violin_staff)

        # make viola voice and staff
        viola_voice = voicetools.Voice(name='Viola Voice')
        viola_staff = stafftools.Staff([viola_voice], name='Viola Staff')
        contexttools.ClefMark('alto')(viola_staff)
        instrumenttools.Viola()(viola_staff)

        # make cello voice and staff
        cello_voice = voicetools.Voice(name='Cello Voice')
        cello_staff = stafftools.Staff([cello_voice], name='Cello Staff')
        contexttools.ClefMark('bass')(cello_staff)
        instrumenttools.Cello(
            short_instrument_name_markup='Vc.'
            )(cello_staff)

        # make bass voice and staff
        bass_voice = voicetools.Voice(name='Bass Voice')
        bass_staff = stafftools.Staff([bass_voice], name='Bass Staff')
        contexttools.ClefMark('bass')(bass_staff)
        instrumenttools.Contrabass(
            short_instrument_name_markup='Cb.'
            )(bass_staff)

        # make strings staff group
        strings_staff_group = scoretools.StaffGroup([
            first_violin_staff,
            second_violin_staff,
            viola_staff,
            cello_staff,
            bass_staff,
            ],
            name='Strings Staff Group',
            )

        # make score
        score = scoretools.Score([
            bell_staff,
            strings_staff_group,
            ],
            name='Pärt Cantus Score'
            )

        # return Pärt Cantus score
        return score




from abjad.tools import componenttools
from abjad.tools import durationtools


def leaf_to_tuplet_with_n_notes_of_equal_written_duration(leaf, n, is_diminution=True):
    '''Change `leaf` to tuplet `n` notes of equal written duration.

    Example 1. Change leaf to augmented tuplet::

        >>> for n in range(1, 11):
        ...     note = Note(0, (3, 16))
        ...     tuplet = tuplettools.leaf_to_tuplet_with_n_notes_of_equal_written_duration(
        ...         note, n, is_diminution=False)
        ...     print tuplet
        ...
        {@ 1:1 c'8. @}
        {@ 1:1 c'16., c'16. @}
        {@ 1:1 c'16, c'16, c'16 @}
        {@ 1:1 c'32., c'32., c'32., c'32. @}
        {@ 5:8 c'64., c'64., c'64., c'64., c'64. @}
        {@ 1:1 c'32, c'32, c'32, c'32, c'32, c'32 @}
        {@ 7:8 c'64., c'64., c'64., c'64., c'64., c'64., c'64. @}
        {@ 1:1 c'64., c'64., c'64., c'64., c'64., c'64., c'64., c'64. @}
        {@ 3:4 c'64, c'64, c'64, c'64, c'64, c'64, c'64, c'64, c'64 @}
        {@ 5:8 c'128., c'128., c'128., c'128., c'128., c'128., c'128., c'128., c'128., c'128. @}

    Example 2. Change to leaf diminished tuplet::

        >>> for n in range(1, 11):
        ...     note = Note(0, (3, 16))
        ...     tuplet = tuplettools.leaf_to_tuplet_with_n_notes_of_equal_written_duration(
        ...         note, n, is_diminution=True)
        ...     print tuplet
        ...
        {@ 1:1 c'8. @}
        {@ 1:1 c'16., c'16. @}
        {@ 1:1 c'16, c'16, c'16 @}
        {@ 1:1 c'32., c'32., c'32., c'32. @}
        {@ 5:4 c'32., c'32., c'32., c'32., c'32. @}
        {@ 1:1 c'32, c'32, c'32, c'32, c'32, c'32 @}
        {@ 7:4 c'32., c'32., c'32., c'32., c'32., c'32., c'32. @}
        {@ 1:1 c'64., c'64., c'64., c'64., c'64., c'64., c'64., c'64. @}
        {@ 3:2 c'32, c'32, c'32, c'32, c'32, c'32, c'32, c'32, c'32 @}
        {@ 5:4 c'64., c'64., c'64., c'64., c'64., c'64., c'64., c'64., c'64., c'64. @}

    Return fixed-duration tuplet.
    '''
    from abjad.tools import notetools
    from abjad.tools import tuplettools

    # find target duration of fixed-duration tuplet
    target_duration = leaf.written_duration

    # find prolated duration of each note in tuplet
    prolated_duration = target_duration / n

    # find written duration of each note in tuplet
    if is_diminution:
        written_duration = prolated_duration.equal_or_greater_assignable
    else:
        written_duration = prolated_duration.equal_or_lesser_assignable

    # make tuplet notes
    notes = n * notetools.Note(0, written_duration)

    # make tuplet
    tuplet = tuplettools.FixedDurationTuplet(target_duration, notes)

    # give leaf position in score structure to tuplet
    componenttools.move_parentage_and_spanners_from_components_to_components([leaf], [tuplet])

    # return tuplet
    return tuplet



import importlib
import inspect
import os
import shutil
import subprocess
import tempfile
from abjad.tools import abctools
from abjad.tools import sequencetools
from abjad.tools import documentationtools
from abjad.tools.abjadbooktools.CodeBlock import CodeBlock
from abjad.tools.abjadbooktools.OutputFormat import OutputFormat


class AbjadBookProcessor(abctools.AbjadObject):

    ### INITIALIZER ###

    def __init__(self, directory, lines, output_format, skip_rendering=False, 
        image_prefix='image', verbose=False):
        assert isinstance(output_format, OutputFormat)
        self._current_code_line = 0
        self._total_code_lines = 0
        self._directory = os.path.abspath(directory)
        self._image_prefix = image_prefix
        self._lines = tuple(lines)
        self._output_format = output_format
        self._skip_rendering = bool(skip_rendering)
        self._verbose = bool(verbose)

    ### SPECIAL METHOD ###

    def __call__(self, verbose=True):

        # Verify input, and extract code blocks
        code_blocks = self._extract_code_blocks(self.lines)        
        for code_block in code_blocks:
            self._total_code_lines += len(code_block.lines)

        # Create a temporary directory, and step into it.
        tmp_directory = self._setup_tmp_directory(self.directory)
        os.chdir(tmp_directory)

        # Process code blocks, and render images inside the temporary directory
        pipe = self._setup_pipe()
        image_count = self._process_code_blocks(pipe, code_blocks, tmp_directory,
            self.image_prefix)
        ly_file_names = self._extract_ly_file_names(code_blocks)
        self._cleanup_pipe(pipe)
        if not self.skip_rendering:
            self._render_ly_files(ly_file_names, self.output_format, verbose)

        # Interleave newly reformatted code with the old, and return.
        if code_blocks:
            result = self._interleave_source_with_code_blocks(
                tmp_directory, self.lines, code_blocks, self.output_format)
        else:
            result = '\n'.join(self.lines)

        # Step out of the tmp directory, back to the original, and cleanup.
        os.chdir(self.directory)
        self._cleanup_image_files(self.directory, tmp_directory, image_count, self.image_prefix,
            self.output_format.image_format)
        self._cleanup_tmp_directory(tmp_directory)

        return result

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def directory(self):
        return self._directory

    @property
    def image_prefix(self):
        return self._image_prefix

    @property
    def lines(self):
        return self._lines

    @property
    def output_format(self):
        return self._output_format

    @property
    def skip_rendering(self):
        return self._skip_rendering

    @property
    def verbose(self):
        return self._verbose

    ### PRIVATE METHODS ###

    def _cleanup_image_files(self, directory, tmp_directory,
        image_count, image_prefix, image_format):
        #print 'CLEANUP IMAGE FILES'
        image_directory = os.path.join(directory, 'images')
        if not os.path.isdir(image_directory):
            os.mkdir(image_directory)
        # remove old images
        #for x in os.listdir(image_directory):
        #    if x.startswith('{}-'.format(image_prefix)) and x.endswith(image_format):
        #        # this should handle both 'index-1.png' and 'index-1-page3.png'
        #        name = os.path.splitext(x)[0]
        #        number = int(name.split('-')[1])
        #        if image_count < number:
        #            os.remove(os.path.join(image_directory, x))
        for x in os.listdir(tmp_directory):
            if x.endswith('.png'):
                source = os.path.join(tmp_directory, x)
                target = os.path.join(image_directory, x)
                if not os.path.exists(target) or \
                    not documentationtools.compare_images(source, target):
                    os.rename(source, target)
                    print '\tMoving {}.'.format(x)
                else:
                    print '\tKeeping old {}.'.format(x)
            elif x.endswith('.pdf'):
                source = os.path.join(tmp_directory, x)
                target = os.path.join(image_directory, x)
                os.rename(source, target)
                print '\tMoving {}.'.format(x)

    def _cleanup_pipe(self, pipe):
        #print 'CLEANUP PIPE'
        pipe.write('quit()\n')
        pipe.close()

    def _cleanup_tmp_directory(self, tmp_directory):
        #print 'CLEANUP TMP DIRECTORY'
        shutil.rmtree(tmp_directory)

    def _extract_code_blocks(self, lines):
        #print 'EXTRACT CODE BLOCKS'
        blocks = []
        block = []
        starting_line_number = 0
        in_block = False
        for i, line in enumerate(lines):
            if line.startswith('<abjad>'):
                if in_block:
                    raise Exception('Extra opening tag at line {}.'.format(i))
                else:
                    in_block = True
                    block = [line]
                    starting_line_number = i

            elif line.startswith('</abjad>'):
                if in_block:
                    in_block = False
                    hide = 'hide=true' in block[0]
                    strip_prompt = 'strip_prompt=true' in block[0]
                    stopping_line_number = i
                    code_block = CodeBlock(
                        block[1:],
                        starting_line_number,
                        stopping_line_number,
                        hide=hide,
                        strip_prompt=strip_prompt
                        )
                    blocks.append(code_block)
                else:
                    raise Exception('Extra closing tag at line {}'.format(i))

            elif in_block:
                block.append(line)

            elif line.startswith('<abjadextract '):
                block = []
                starting_line_number = stopping_line_number = i
                hide = 'hide=true' in line
                strip_prompt = 'strip_prompt=true' in line
                code_address = line.partition('<abjadextract ')[-1].partition(' \>')[0] 
                module_name, dot, attr_name = code_address.rpartition('.')
                module = importlib.import_module(module_name)
                attr = getattr(module, attr_name)
                code_lines = inspect.getsource(attr).splitlines()
                code_block = CodeBlock(
                    code_lines,
                    starting_line_number,
                    stopping_line_number,
                    hide=hide,
                    strip_prompt=strip_prompt
                    )
                blocks.append(code_block)

        if in_block:
            raise Exception('Unterminated tag at EOF.')

        return tuple(blocks)

    def _extract_ly_file_names(self, code_blocks):
        #print 'EXTRACT LY FILENAMES'
        file_names = []
        for code_block in code_blocks:
            for result in code_block.processed_results:
                if isinstance(result, dict):
                    file_names.append(result['file_name'])
        return file_names

    def _interleave_source_with_code_blocks(self, tmp_directory, lines, code_blocks, output_format):

        #print 'INTERLEAVE SOURCE WITH CODE BLOCKS'
        image_file_names = [x for x in os.listdir(tmp_directory)
            if (x.endswith(output_format.image_format) and
               x.startswith(self.image_prefix))]

        image_dict = {}
        for image_filename in image_file_names:
            suffix = os.path.splitext(image_filename.partition('-')[2])[0]
            index, part, page = suffix.partition('-')
            index = int(index)
            if page:
                page = int(page.strip('page'))
            else:
                page = 0
            if index not in image_dict:
                image_dict[index] = {}
            image_dict[index][page] = image_filename

        interleaved = []
        interleaved.append('\n'.join(lines[:code_blocks[0].starting_line_number]))
        for pair in sequencetools.iterate_sequence_pairwise_strict(code_blocks):
            first_block, second_block = pair
            interleaved.extend(output_format(first_block, image_dict))
            interleaved.append('\n'.join(lines[
                first_block.ending_line_number + 1:second_block.starting_line_number]))

        interleaved.extend(output_format(code_blocks[-1], image_dict))
        interleaved.append('\n'.join(lines[code_blocks[-1].ending_line_number + 1:]))
        return '\n'.join(interleaved)

    def _process_code_blocks(self, pipe, code_blocks, directory, image_prefix):
        #print 'PROCESS CODE BLOCKS'
        image_count = 0
        for i, code_block in enumerate(code_blocks):
            #print '\tCODE BLOCK', i
            image_count = code_block(self, pipe, image_count, directory, image_prefix, verbose=self.verbose)
        return image_count

    def _render_ly_files(self, file_names, output_format, verbose):
        #print 'RENDER LY FILES'
        for file_name in file_names:
            print '\tRendering {}.ly...'.format(file_name)
            try:
                if output_format.image_format == 'pdf':
                    command = 'lilypond {}.ly'.format(file_name)
                    print '\t\t{}'.format(command)
                    self._run_command(command, verbose)
                    command = 'pdfcrop {}.pdf {}.pdf'.format(file_name, file_name)
                    print '\t\t{}'.format(command)
                    self._run_command(command, verbose)
                elif output_format.image_format == 'png':
                    command = 'lilypond --png -dresolution=300 {}.ly'.format(file_name)
                    print '\t\t{}'.format(command)
                    assert os.path.exists('{}.ly'.format(file_name))
                    self._run_command(command, verbose)
                    for file in os.listdir('.'):
                        if file.startswith(file_name) and file.endswith('.png'):
                            command = 'convert {} -trim -resample 40%% {}'.format(file, file)
                            print '\t\t{}'.format(command)
                            self._run_command(command, verbose)
            except AssertionError, e:
                print e

    def _run_command(self, command, verbose):
        if verbose:
            subprocess.call(command, shell=True)
        else:
            subprocess.call(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    def _setup_pipe(self):
        #print 'SETUP PIPE'
        pipe = documentationtools.Pipe()
        pipe.read_wait()
        pipe.write('from abjad import *\n')
        pipe.read_wait()
        return pipe

    def _setup_tmp_directory(self, directory):
        #print 'SETUP TMP DIRECTORY'
        tmp_directory = os.path.abspath(tempfile.mkdtemp(dir=directory))
        return tmp_directory

    ### PUBLIC METHODS ###

    def update_status(self, line):
        self._current_code_line += 1
        percentage = float(self._current_code_line) / self._total_code_lines
        message = '[{:4.0%}] {}'.format(percentage, line)
        print message





from abjad.tools import componenttools


# TODO: optimize without full-component traversal.
def get_vertical_moment_starting_with_component(component, governor=None):
    r'''.. versionadded:: 2.0

    Get vertical moment starting with `component`::

        >>> score = Score([])
        >>> staff = Staff(r"\times 4/3 { d''8 c''8 b'8 }")
        >>> score.append(staff)

    ::

        >>> piano_staff = scoretools.PianoStaff([])
        >>> piano_staff.append(Staff("a'4 g'4"))
        >>> piano_staff.append(Staff(r"""\clef "bass" f'8 e'8 d'8 c'8"""))
        >>> score.append(piano_staff)

    ::

        >>> f(score)
        \new Score <<
            \new Staff {
                \fraction \times 4/3 {
                    d''8
                    c''8
                    b'8
                }
            }
            \new PianoStaff <<
                \new Staff {
                    a'4
                    g'4
                }
                \new Staff {
                    \clef "bass"
                    f'8
                    e'8
                    d'8
                    c'8
                }
            >>
        >>

    ::

        >>> leaf = piano_staff[1][1]

    ::

        >>> verticalitytools.get_vertical_moment_starting_with_component(leaf)
        VerticalMoment(1/8, <<3>>)

    Get vertical moment starting with `component` in `governor` when
    `governor` is not none::

        >>> verticalitytools.get_vertical_moment_starting_with_component(leaf, 
        ... governor=piano_staff)
        VerticalMoment(1/8, <<2>>)

    Return vertical moment.
    '''
    from abjad.tools import verticalitytools

    offset = component.start_offset

    if governor is None:
        governor = component.parentage.root

    return verticalitytools.get_vertical_moment_at_offset_in_expr(governor, offset)



from abjad.tools import documentationtools
import inspect
import os


def get_developer_script_classes():
    '''Return a list of all developer script classes.'''

    from abjad.tools import developerscripttools
    from abjad.tools import abjadbooktools

    tools_package_paths = [
        abjadbooktools.__path__[0],
        developerscripttools.__path__[0]
    ]
    script_classes = []
    for tools_package_path in tools_package_paths:
        klasses = documentationtools.ClassCrawler(tools_package_path,
            root_package_name='abjad')()
        for klass in klasses:
            if developerscripttools.DeveloperScript in inspect.getmro(klass) and \
                not inspect.isabstract(klass):
                script_classes.append(klass)

    return list(sorted(script_classes, key=lambda x: x.__name__))



def partition_sequence_by_weights_exactly(sequence, weights, cyclic=False, overhang=False):
    r'''.. versionadded:: 1.1

    Partition `sequence` by `weights` exactly.

        >>> sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5]

    Example 1. Partition sequence once by weights exactly without overhang::

        >>> sequencetools.partition_sequence_by_weights_exactly(
        ...     sequence, [3, 9], cyclic=False, overhang=False)
        [[3], [3, 3, 3]]

    Example 2. Partition sequence once by weights exactly with overhang::

        >>> sequencetools.partition_sequence_by_weights_exactly(
        ...     sequence, [3, 9], cyclic=False, overhang=True)
        [[3], [3, 3, 3], [4, 4, 4, 4, 5]]

    Example 3. Partition sequence cyclically by weights exactly without overhang::

        >>> sequencetools.partition_sequence_by_weights_exactly(
        ...     sequence, [12], cyclic=True, overhang=False)
        [[3, 3, 3, 3], [4, 4, 4]]

    Example 4. Partition sequence cyclically by weights exactly with overhang::

        >>> sequencetools.partition_sequence_by_weights_exactly(
        ...     sequence, [12], cyclic=True, overhang=True)
        [[3, 3, 3, 3], [4, 4, 4], [4, 5]]

    Return list sequence objects.
    '''
    from abjad.tools import sequencetools

    candidate = sequencetools.split_sequence_by_weights(sequence, weights, cyclic=cyclic, overhang=overhang)
    flattened_candidate = sequencetools.flatten_sequence(candidate)
    if flattened_candidate == sequence[:len(flattened_candidate)]:
        return candidate
    else:
        raise PartitionError('can not partition exactly.')



from abjad.tools.abctools import AbjadObject


class Scheme(AbjadObject):
    r'''Abjad model of Scheme code::

        >>> s = schemetools.Scheme(True)
        >>> s.lilypond_format
        '##t'

    schemetools.Scheme can represent nested structures:

    ::

        >>> s = schemetools.Scheme(('left', (1, 2, False)), ('right', (1, 2, 3.3)))
        >>> s.lilypond_format
        '#((left (1 2 #f)) (right (1 2 3.3)))'

    schemetools.Scheme wraps variable-length arguments into a tuple:

    ::

        >>> s = schemetools.Scheme(1, 2, 3)
        >>> q = schemetools.Scheme((1, 2, 3))
        >>> s.lilypond_format == q.lilypond_format
        True

    schemetools.Scheme also takes an optional `quoting` keyword, by which Scheme's
    various quote, unquote, unquote-splicing characters can be prepended to the
    formatted result:

    ::

        >>> s = schemetools.Scheme((1, 2, 3), quoting="'#")
        >>> s.lilypond_format
        "#'#(1 2 3)"

    schemetools.Scheme can also force quotes around strings which contain no whitespace:

    ::

        >>> s = schemetools.Scheme('nospaces', force_quotes=True)
        >>> f(s)
        #"nospaces"

    The above is useful in certain \override situations, as LilyPond's Scheme interpreter
    will treat unquoted strings as symbols rather than strings.

    Scheme is immutable.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_force_quotes', '_quoting', '_value',)

    ### INITIALIZER ###

    def __init__(self, *args, **kwargs):
        if 1 == len(args):
            if isinstance(args[0], type(self)):
                args = args[0]._value
            else:
                args = args[0]
        quoting = kwargs.get('quoting')
        force_quotes = bool(kwargs.get('force_quotes'))
        assert isinstance(quoting, (str, type(None)))
        if quoting is not None:
            assert all([x in ("'", ',', '@', '`', '#') for x in quoting])
        self._force_quotes = force_quotes
        self._quoting = quoting
        self._value = args

    ### SPECIAL METHODS ###

    def __eq__(self, other):
        if type(self) == type(other):
            if self._value == other._value:
                return True
        return False

    def __getnewargs__(self):
        return (self._value,)

    def __repr__(self):
        return "%s(%r)" % (type(self).__name__, self._value)

    def __str__(self):
        if self._quoting is not None:
            return self._quoting + self._formatted_value
        return self._formatted_value

    ### PRIVATE PROPERTIES ###

    @property
    def _formatted_value(self):
        from abjad.tools import schemetools
        return schemetools.format_scheme_value(self._value, force_quotes=self.force_quotes)

    ### PUBLIC PROPERTIES ###

    @property
    def force_quotes(self):
        return self._force_quotes

    @property
    def lilypond_format(self):
        '''Hash-mark-prepended format of Scheme::

            >>> from abjad.tools.schemetools import Scheme
            >>> Scheme(True).lilypond_format
            '##t'

        Returns string.
        '''
        if self._quoting is not None:
            return '#' + self._quoting + self._formatted_value
        return '#%s' % self._formatted_value



from abjad import *


def test_pitchtools_calculate_harmonic_counterpoint_interval_class_01():
    '''Ascending intervals greater than an octave.'''

    mcpi = pitchtools.calculate_melodic_counterpoint_interval_class(
        pitchtools.NamedChromaticPitch(-3), pitchtools.NamedChromaticPitch(12))
    assert mcpi == pitchtools.MelodicCounterpointIntervalClass(3)

    mcpi = pitchtools.calculate_melodic_counterpoint_interval_class(
        pitchtools.NamedChromaticPitch(-2), pitchtools.NamedChromaticPitch(12))
    assert mcpi == pitchtools.MelodicCounterpointIntervalClass(2)

    mcpi = pitchtools.calculate_melodic_counterpoint_interval_class(
        pitchtools.NamedChromaticPitch(-1), pitchtools.NamedChromaticPitch(12))
    assert mcpi == pitchtools.MelodicCounterpointIntervalClass(2)


def test_pitchtools_calculate_harmonic_counterpoint_interval_class_02():
    '''Ascending octave.'''

    mcpi = pitchtools.calculate_melodic_counterpoint_interval_class(
        pitchtools.NamedChromaticPitch(0), pitchtools.NamedChromaticPitch(12))
    assert mcpi == pitchtools.MelodicCounterpointIntervalClass(8)


def test_pitchtools_calculate_harmonic_counterpoint_interval_class_03():
    '''Ascending intervals less than an octave.'''

    mcpi = pitchtools.calculate_melodic_counterpoint_interval_class(
        pitchtools.NamedChromaticPitch(9), pitchtools.NamedChromaticPitch(12))
    assert mcpi == pitchtools.MelodicCounterpointIntervalClass(3)

    mcpi = pitchtools.calculate_melodic_counterpoint_interval_class(
        pitchtools.NamedChromaticPitch(10), pitchtools.NamedChromaticPitch(12))
    assert mcpi == pitchtools.MelodicCounterpointIntervalClass(2)

    mcpi = pitchtools.calculate_melodic_counterpoint_interval_class(
        pitchtools.NamedChromaticPitch(11), pitchtools.NamedChromaticPitch(12))
    assert mcpi == pitchtools.MelodicCounterpointIntervalClass(2)


def test_pitchtools_calculate_harmonic_counterpoint_interval_class_04():
    '''Unison.'''

    mcpi = pitchtools.calculate_melodic_counterpoint_interval_class(
        pitchtools.NamedChromaticPitch(12), pitchtools.NamedChromaticPitch(12))
    assert mcpi == pitchtools.MelodicCounterpointIntervalClass(1)


def test_pitchtools_calculate_harmonic_counterpoint_interval_class_05():
    '''Descending intervals greater than an octave.'''

    mcpi = pitchtools.calculate_melodic_counterpoint_interval_class(
        pitchtools.NamedChromaticPitch(12), pitchtools.NamedChromaticPitch(-3))
    assert mcpi == pitchtools.MelodicCounterpointIntervalClass(-3)

    mcpi = pitchtools.calculate_melodic_counterpoint_interval_class(
        pitchtools.NamedChromaticPitch(12), pitchtools.NamedChromaticPitch(-2))
    assert mcpi == pitchtools.MelodicCounterpointIntervalClass(-2)

    mcpi = pitchtools.calculate_melodic_counterpoint_interval_class(
        pitchtools.NamedChromaticPitch(12), pitchtools.NamedChromaticPitch(-1))
    assert mcpi == pitchtools.MelodicCounterpointIntervalClass(-2)


def test_pitchtools_calculate_harmonic_counterpoint_interval_class_06():
    '''Descending octave.'''

    mcpi = pitchtools.calculate_melodic_counterpoint_interval_class(
        pitchtools.NamedChromaticPitch(12), pitchtools.NamedChromaticPitch(0))
    assert mcpi == pitchtools.MelodicCounterpointIntervalClass(-8)


def test_pitchtools_calculate_harmonic_counterpoint_interval_class_07():
    '''Descending intervals less than an octave.'''

    mcpi = pitchtools.calculate_melodic_counterpoint_interval_class(
        pitchtools.NamedChromaticPitch(12), pitchtools.NamedChromaticPitch(9))
    assert mcpi == pitchtools.MelodicCounterpointIntervalClass(-3)

    mcpi = pitchtools.calculate_melodic_counterpoint_interval_class(
        pitchtools.NamedChromaticPitch(12), pitchtools.NamedChromaticPitch(10))
    assert mcpi == pitchtools.MelodicCounterpointIntervalClass(-2)

    mcpi = pitchtools.calculate_melodic_counterpoint_interval_class(
        pitchtools.NamedChromaticPitch(12), pitchtools.NamedChromaticPitch(11))
    assert mcpi == pitchtools.MelodicCounterpointIntervalClass(-2)



from abjad import *


def test_TreeContainer_insert_01():

    leaf_a = datastructuretools.TreeNode()
    leaf_b = datastructuretools.TreeNode()
    leaf_c = datastructuretools.TreeNode()

    container = datastructuretools.TreeContainer()
    assert container.children == ()

    container.insert(0, leaf_a)
    assert container.children == (leaf_a,)

    container.insert(0, leaf_b)
    assert container.children == (leaf_b, leaf_a)

    container.insert(1, leaf_c)
    assert container.children == (leaf_b, leaf_c, leaf_a)



from abjad import *


def test_tietools_iterate_pitched_tie_chains_in_expr_01():

    staff = Staff('c ~ c r d ~ d r')

    tie_chains = list(tietools.iterate_pitched_tie_chains_in_expr(staff, reverse=True))

    assert tie_chains[0] == tietools.TieChain(staff[3:5])
    assert tie_chains[1] == tietools.TieChain(staff[:2])


def test_tietools_iterate_pitched_tie_chains_in_expr_02():

    staff = Staff('c ~ c r d ~ d r')
    
    tie_chains = list(tietools.iterate_pitched_tie_chains_in_expr(staff))

    assert tie_chains[0] == tietools.TieChain(staff[:2])
    assert tie_chains[1] == tietools.TieChain(staff[3:5])



from abjad import *


def test_containertools_insert_component_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    beamtools.BeamSpanner(staff.leaves)
    containertools.insert_component(staff, 1, Note("cs'8"), fracture_spanners=False)

    r'''
    \new Staff {
        c'8 [
        cs'8
        d'8
        e'8
        f'8 ]
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert staff.lilypond_format == "\\new Staff {\n\tc'8 [\n\tcs'8\n\td'8\n\te'8\n\tf'8 ]\n}"


def test_containertools_insert_component_02():
    '''Insert component into container at index i.
    Fracture spanners to the left of index i.
    Fracture spanners to the right of index i.
    Return Python list of fractured spanners.
    '''

    "Insert works just before a spanner."

    t = Staff([Note(n, (1, 8)) for n in range(4)])
    beamtools.BeamSpanner(t[:])
    containertools.insert_component(t, 0, Rest((1, 4)), fracture_spanners=True)

    r'''
    \new Staff {
        r4
        c'8 [
        cs'8
        d'8
        ef'8 ]
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Staff {\n\tr4\n\tc'8 [\n\tcs'8\n\td'8\n\tef'8 ]\n}"


def test_containertools_insert_component_03():
    '''Insert works inside a spanner.
    '''

    t = Staff([Note(n, (1, 8)) for n in range(4)])
    beamtools.BeamSpanner(t[:])
    containertools.insert_component(t, 1, Rest((1, 4)), fracture_spanners=True)

    r'''
    \new Staff {
        c'8 [ ]
        r4
        cs'8 [
        d'8
        ef'8 ]
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Staff {\n\tc'8 [ ]\n\tr4\n\tcs'8 [\n\td'8\n\tef'8 ]\n}"


def test_containertools_insert_component_04():
    '''Insert works just after a spanner.
    '''

    t = Staff([Note(n, (1, 8)) for n in range(4)])
    beamtools.BeamSpanner(t[:])
    containertools.insert_component(t, 4, Rest((1, 4)), fracture_spanners=True)

    r'''
    \new Staff {
        c'8 [
        cs'8
        d'8
        ef'8 ]
        r4
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Staff {\n\tc'8 [\n\tcs'8\n\td'8\n\tef'8 ]\n\tr4\n}"


def test_containertools_insert_component_05():
    '''Insert works with really big positive values.'''

    t = Staff([Note(n, (1, 8)) for n in range(4)])
    beamtools.BeamSpanner(t[:])
    containertools.insert_component(t, 1000, Rest((1, 4)), fracture_spanners=True)

    r'''
    \new Staff {
        c'8 [
        cs'8
        d'8
        ef'8 ]
    r4
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Staff {\n\tc'8 [\n\tcs'8\n\td'8\n\tef'8 ]\n\tr4\n}"


def test_containertools_insert_component_06():
    '''Insert works with negative values.
    '''

    t = Staff([Note(n, (1, 8)) for n in range(4)])
    beamtools.BeamSpanner(t[:])
    containertools.insert_component(t, -1, Rest((1, 4)), fracture_spanners=True)

    r'''
    \new Staff {
        c'8 [
        cs'8
        d'8 ]
        r4
        ef'8 [ ]
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Staff {\n\tc'8 [\n\tcs'8\n\td'8 ]\n\tr4\n\tef'8 [ ]\n}"


def test_containertools_insert_component_07():
    '''Insert works with really big negative values.
    '''

    t = Staff([Note(n, (1, 8)) for n in range(4)])
    beamtools.BeamSpanner(t[:])
    containertools.insert_component(t, -1000, Rest((1, 4)), fracture_spanners=True)

    r'''
    \new Staff {
        r4
        c'8 [
        cs'8
        d'8
        ef'8 ]
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Staff {\n\tr4\n\tc'8 [\n\tcs'8\n\td'8\n\tef'8 ]\n}"


def test_containertools_insert_component_08():
    '''Inserting a note from one container into another container
    switches note parent from first container to second.
    '''

    v = Voice("c'8 d'8 e'8 f'8")
    t = Staff(notetools.make_repeated_notes(8))
    note = v[0]
    containertools.insert_component(t, 1, v[0], fracture_spanners=True)

    assert wellformednesstools.is_well_formed_component(v)
    assert wellformednesstools.is_well_formed_component(t)
    assert not note in v
    assert note._parent is t



from abjad.tools import spannertools


def apply_durated_complex_beam_spanner_to_measures(measures):
    r'''.. versionadded:: 1.1

    Apply durated complex beam spanner to `measures`::

        >>> staff = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 2)
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(staff)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> measures = staff[:]
        >>> beamtools.apply_durated_complex_beam_spanner_to_measures(measures)
        DuratedComplexBeamSpanner(|2/8(2)|, |2/8(2)|)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                \set stemLeftBeamCount = #0
                \set stemRightBeamCount = #1
                c'8 [
                \set stemLeftBeamCount = #1
                \set stemRightBeamCount = #1
                d'8
            }
            {
                \set stemLeftBeamCount = #1
                \set stemRightBeamCount = #1
                e'8
                \set stemLeftBeamCount = #1
                \set stemRightBeamCount = #0
                f'8 ]
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Set beam spanner durations to preprolated measure durations.

    Return beam spanner created.

    .. versionchanged:: 2.0
        renamed ``measuretools.beam_together()``.

    .. versionchanged:: 2.9
        renamed ``measuretools.apply_durated_complex_beam_spanner_to_measures()`` to
        ``beamtools.apply_durated_complex_beam_spanner_to_measures()``.
    '''
    from abjad.tools import beamtools

    durations = []
    for measure in measures:
        spannertools.destroy_spanners_attached_to_component(measure, beamtools.BeamSpanner)
        durations.append(measure.preprolated_duration)
    beam = beamtools.DuratedComplexBeamSpanner(measures, durations=durations, span=1)
    return beam



from abjad import *


def apply_page_breaks(score):

    bell_voice = score['Bell Voice']

    measure_indices = [5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 72,
        79, 86, 93, 100]

    for measure_index in measure_indices:
        marktools.LilyPondCommandMark(
            'break', 
            'after'
            )(bell_voice[measure_index])



def get_spanners_attached_to_any_proper_child_of_component(component, klass=None):
    r'''.. versionadded:: 2.0

    Get all spanners attached to any proper children of `component`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> beam = beamtools.BeamSpanner(staff.leaves)
        >>> first_slur = spannertools.SlurSpanner(staff.leaves[:2])
        >>> second_slur = spannertools.SlurSpanner(staff.leaves[2:])
        >>> trill = spannertools.TrillSpanner(staff)

    ::

        >>> f(staff)
        \new Staff {
            c'8 [ ( \startTrillSpan
            d'8 )
            e'8 (
            f'8 ] ) \stopTrillSpan
        }

    ::

        >>> len(spannertools.get_spanners_attached_to_any_proper_child_of_component(
        ...     staff))
        3

    Get all spanners of `klass` attached to any proper children of `component`::

        >>> spanner_klass = spannertools.SlurSpanner
        >>> result = spannertools.get_spanners_attached_to_any_proper_child_of_component(
        ...     staff, spanner_klass)

    ::
        
        >>> list(sorted(result))
        [SlurSpanner(c'8, d'8), SlurSpanner(e'8, f'8)]

    Get all spanners of any `klass` attached to any proper children of `component`::

        >>> spanner_klasses = (spannertools.SlurSpanner, beamtools.BeamSpanner)
        >>> result = spannertools.get_spanners_attached_to_any_proper_child_of_component(
        ...     staff, spanner_klasses)

    ::
        
        >>> list(sorted(result))
        [BeamSpanner(c'8, d'8, e'8, f'8), SlurSpanner(c'8, d'8), SlurSpanner(e'8, f'8)]

    Return unordered set of zero or more spanners.

    .. versionchanged:: 2.0
        renamed ``spannertools.get_all_spanners_attached_to_any_proper_children_of_component()`` to
        ``spannertools.get_spanners_attached_to_any_proper_child_of_component()``.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import spannertools

    # note: externalization of (old) component spanner aggregator 'children' property
    result = set([])
    components = iterationtools.iterate_components_in_expr(component)

    # remove component itself from iteration
    components.next()

    # iterate only proper children of component and save spanners
    for component in components:
        result.update(spannertools.get_spanners_attached_to_component(component, klass))

    # return result
    return result
